# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2017 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from datetime import date

from django.http import JsonResponse
from django.urls import resolve
from django.utils.decorators import method_decorator
from django.views.decorators.clickjacking import xframe_options_exempt
from django.views.generic.base import TemplateView
from django.views.generic.list import ListView

from rolepermissions.mixins import HasPermissionsMixin
from stronghold.decorators import public
from stronghold.views import StrongholdPublicMixin

from apps.challenge.models.session import Session
from apps.common import DV_SEASON_AUTUMN, DV_SEASON_SPRING
from apps.common.views import ExportMixin, PaginatorMixin
from defivelo.views.common import MenuView

from . import utils
from .exports import (
    LogisticsExport,
    QualifsCalendarExport,
    SeasonSessionsMixin,
    SeasonStatsExport,
)
from .forms import CantonFilterForm


class SessionsPublicView(StrongholdPublicMixin):
    def get_queryset(self):
        # This needs to be a get_queryset to make sure the date.today() is evaluated at view time, not at start time.
        return (
            Session.objects.filter(day__gte=date.today())
            .order_by("day", "orga")
            .prefetch_related("orga")
        )

    @method_decorator(public)
    @method_decorator(xframe_options_exempt)
    def dispatch(self, *args, **kwargs):
        return super(SessionsPublicView, self).dispatch(*args, **kwargs)


class NextQualifs(SessionsPublicView, PaginatorMixin, ListView):
    template_name = "info/next_qualifs.html"
    paginate_by = 8
    context_object_name = "sessions"


class JSONNextQualifs(SessionsPublicView, TemplateView):
    def render_to_response(self, context, **response_kwargs):
        sessions = []
        for session in self.get_queryset().all():
            session_representation = {
                "canton": session.orga.address_canton,
                "date": session.day,
                "begin": session.begin,
                "orga": {"name": session.orga.name, "abbr": session.orga.abbr},
                "city": session.city,
            }
            sessions.append(session_representation)
        return JsonResponse({"sessions": sessions}, **response_kwargs)


class SeasonExportsMixin(MenuView):
    def dispatch(self, *args, **kwargs):
        try:
            self.export_year = int(kwargs.pop("year"))
        except (TypeError, KeyError):
            self.export_year = date.today().year
        try:
            self.export_season = int(kwargs.pop("dv_season"))
        except (TypeError, KeyError):
            self.export_season = self.current_season()
        return super(SeasonExportsMixin, self).dispatch(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(SeasonExportsMixin, self).get_context_data(**kwargs)
        context["export_period"] = {
            "year": self.export_year,
            "season": self.export_season,
        }
        context["previous_period"] = {
            "year": self.export_year
            - (1 if self.export_season == DV_SEASON_SPRING else 0),
            "season": DV_SEASON_AUTUMN
            if self.export_season == DV_SEASON_SPRING
            else DV_SEASON_SPRING,
        }
        context["next_period"] = {
            "year": self.export_year
            + (1 if self.export_season == DV_SEASON_AUTUMN else 0),
            "season": DV_SEASON_AUTUMN
            if self.export_season == DV_SEASON_SPRING
            else DV_SEASON_SPRING,
        }
        context["nav_url"] = resolve(self.request.path).url_name
        context["dataset_exporturl"] = context["nav_url"] + "-export"
        context["menu_category"] = "statistics"
        context["submenu_category"] = "statistics-season"
        return context


class IfDatasetExportMixin(object):
    def get_context_data(self, **kwargs):
        context = super(IfDatasetExportMixin, self).get_context_data(**kwargs)
        try:
            context["dataset"] = self.get_dataset(html=True)
            context["dataset_title"] = self.get_dataset_title()
        except AttributeError:
            pass
        return context


class SeasonExports(
    SeasonExportsMixin, HasPermissionsMixin, IfDatasetExportMixin, TemplateView
):
    required_permission = "challenge_season_crud"
    template_name = "info/season_exports.html"

    def get_context_data(self, **kwargs):
        context = super(SeasonExports, self).get_context_data(**kwargs)
        context["submenu_category"] = "statistics-season"
        return context


class QualifsCalendarView(SeasonSessionsMixin, SeasonExportsMixin, ListView):
    limit_to_my_cantons = False
    template_name = "info/qualifs_calendar.html"
    context_object_name = "sessions"

    def post(self, request, *args, **kwargs):
        form = CantonFilterForm(request.POST)
        if form.is_valid():
            self.cantons = form.cleaned_data["canton"]
        return self.get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(QualifsCalendarView, self).get_context_data(**kwargs)
        context["menu_category"] = "season"
        context["submenu_category"] = "qualifs-calendar"
        # Add the form for picking a new helper
        context["form"] = CantonFilterForm()
        our_sessions = context["sessions"]
        if hasattr(self, "cantons") and len(self.cantons) > 0:
            our_sessions = our_sessions.filter(orga__address_canton__in=self.cantons)
        our_sessions = list(our_sessions)

        if len(our_sessions) == 0:
            return context

        context["legend_cantons"] = {s.orga.address_canton: "" for s in our_sessions}

        context["date_sessions"] = utils._get_qualifs_calendar_structured_data(
            our_sessions
        )

        return context


class QualifsCalendarExportView(
    QualifsCalendarExport, SeasonExportsMixin, ExportMixin, ListView
):
    def get_context_data(self, **kwargs):
        context = super(QualifsCalendarExportView, self).get_context_data(**kwargs)
        our_sessions = context["session_list"]
        if hasattr(self, "cantons") and len(self.cantons) > 0:
            our_sessions = our_sessions.filter(orga__address_canton__in=self.cantons)
        our_sessions = list(our_sessions)

        if len(our_sessions) == 0:
            return context

        context["legend_cantons"] = {s.orga.address_canton: "" for s in our_sessions}
        context["date_sessions"] = utils._get_qualifs_calendar_structured_data(
            our_sessions
        )

        return context


class SeasonStatsView(SeasonStatsExport, SeasonExports):
    pass


class SeasonStatsExportView(
    SeasonStatsExport, SeasonExportsMixin, ExportMixin, ListView
):
    pass


class LogisticsView(LogisticsExport, SeasonExports):
    pass


class LogisticsExportView(LogisticsExport, SeasonExportsMixin, ExportMixin, ListView):
    pass



# Generated by Django 2.2.9 on 2020-02-24 15:11

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0050_userprofile_bank_name"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="cresus_employee_number",
            field=models.CharField(
                blank=True, max_length=63, verbose_name="Numéro d'employé Crésus"
            ),
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2017 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import include
from django.urls import re_path
from django.views.decorators.cache import never_cache

from .views import (
    JSONNextQualifs,
    LogisticsExportView,
    LogisticsView,
    NextQualifs,
    QualifsCalendarExportView,
    QualifsCalendarView,
    SeasonExports,
    SeasonStatsExportView,
    SeasonStatsView,
)

urlpatterns = [
    re_path(
        r"^qualifs/$", never_cache(NextQualifs.as_view()), name="public-nextqualifs"
    ),
    re_path(
        r"^qualifs.json$",
        never_cache(JSONNextQualifs.as_view()),
        name="public-json-nextqualifs",
    ),
    re_path(
        r"^(?:(?P<year>[0-9]{4})_(?P<dv_season>[0-9]+)/)?",
        include(
            [
                re_path(
                    r"^$", never_cache(SeasonExports.as_view()), name="season-exports"
                ),
                # Calendar
                re_path(
                    r"^calendar/$",
                    never_cache(QualifsCalendarView.as_view()),
                    name="qualifs-calendar",
                ),
                re_path(
                    r"^calendar-(?P<format>[a-z]+)$",
                    never_cache(QualifsCalendarExportView.as_view()),
                    name="qualifs-calendar-export",
                ),
                # Statistiques de Mois
                re_path(
                    r"^stats/$",
                    never_cache(SeasonStatsView.as_view()),
                    name="season-stats",
                ),
                re_path(
                    r"^stats-(?P<format>[a-z]+)$",
                    never_cache(SeasonStatsExportView.as_view()),
                    name="season-stats-export",
                ),
                # Planification Logistique
                re_path(
                    r"^logistics/$",
                    never_cache(LogisticsView.as_view()),
                    name="logistics",
                ),
                re_path(
                    r"^logistics-(?P<format>[a-z]+)$",
                    never_cache(LogisticsExportView.as_view()),
                    name="logistics-export",
                ),
            ]
        ),
    ),
]



from __future__ import unicode_literals

import localflavor.generic.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("auth", "0006_require_contenttypes_0002"),
    ]

    operations = [
        migrations.CreateModel(
            name="UserProfile",
            fields=[
                (
                    "user",
                    models.OneToOneField(
                        primary_key=True,
                        serialize=False,
                        to=settings.AUTH_USER_MODEL,
                        related_name="profile",
                        on_delete=models.CASCADE,
                    ),
                ),
                ("iban", localflavor.generic.models.IBANField(max_length=34)),
            ],
        ),
    ]



import datetime

from django.db import IntegrityError
from django.test import TestCase
from django.urls import reverse

from apps.challenge.tests.factories import QualificationFactory, SessionFactory
from apps.common import DV_STATES
from apps.orga.tests.factories import OrganizationFactory
from apps.user.tests.factories import UserFactory
from defivelo.tests.utils import AuthClient, PowerUserAuthClient, StateManagerAuthClient

from .. import timesheets_overview
from ..models import MonthlyCantonalValidation, MonthlyCantonalValidationUrl
from .factories import MonthlyCantonalValidationFactory, TimesheetFactory


class MCVTestCase(TestCase):
    def test_no_two_identical_month_canton(self):
        today = datetime.datetime.today().replace(day=4)
        mcv = MonthlyCantonalValidationFactory(canton="VD", date=today)
        # Day is reset to start of the month; day=1
        self.assertEqual(mcv.date.day, 1, mcv)

        with self.assertRaises(IntegrityError):
            MonthlyCantonalValidationFactory(canton="VD", date=today.replace(day=8))


class AuthUserTest(TestCase):
    def setUp(self):
        self.client = AuthClient()
        super().setUp()

    def test_no_access_to_mcvs_lists(self):
        today = datetime.datetime.today()
        url = reverse(
            "salary:validations-month",
            kwargs={"month": today.month, "year": today.year},
        )
        # Final URL is forbidden
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 403, url)
        url = reverse(
            "salary:validations-year",
            kwargs={"year": today.year},
        )
        # Final URL is forbidden
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 403, url)

    def test_no_access_to_mcvs_updates(self):
        today = datetime.datetime.today()
        for canton in DV_STATES:
            url = reverse(
                "salary:validation-update",
                kwargs={"month": today.month, "year": today.year, "canton": canton},
            )
            # Final URL is forbidden
            response = self.client.get(url, follow=True)
            self.assertEqual(response.status_code, 403, url)


class StateManagerUserTest(TestCase):
    def setUp(self):
        self.client = StateManagerAuthClient()
        # Our migrations create two for us
        self.urls = MonthlyCantonalValidationUrl.objects.all()
        super().setUp()

    def test_access_to_mcvs_lists(self):
        today = datetime.datetime.today()
        url = reverse(
            "salary:validations-month",
            kwargs={"month": today.month, "year": today.year},
        )
        # Final URL is accessible
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200, url)

        url = reverse(
            "salary:validations-year",
            kwargs={"year": today.year},
        )
        # Final URL is accessible
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200, url)

    def test_access_to_my_mcvs_updates(self):
        today = datetime.datetime.today()
        managed_cantons = [c.canton for c in self.client.user.managedstates.all()]
        for canton in DV_STATES:
            mcv_kwargs = {"month": today.month, "year": today.year, "canton": canton}
            url = reverse("salary:validation-update", kwargs=mcv_kwargs)
            mcv_gs = MonthlyCantonalValidation.objects.filter(
                canton=canton, date=today.replace(day=1)
            )
            # They don't exist before we access them
            self.assertFalse(mcv_gs.exists())
            # Final URL is accessible if in my cantons
            response = self.client.get(url, follow=True)
            if canton in managed_cantons:
                self.assertEqual(response.status_code, 200, url)
                # We looked at it, it exists now
                self.assertTrue(mcv_gs.exists())
                # Try to validate without ticking anything
                response = self.client.post(url, {})
                self.assertEqual(response.status_code, 302, url)

                # Get the object, and assert that it didn't get validated for real
                mcv = mcv_gs.first()
                self.assertFalse(mcv.validated, mcv)

                # Try to validate with the urls ticked only
                initial = {}
                for u in self.urls:
                    initial[f"url_{u.pk}"] = True
                response = self.client.post(url, initial)
                self.assertEqual(response.status_code, 302, url)

                # Get the object, and assert that it didn't get validated for real
                mcv = mcv_gs.first()
                self.assertFalse(mcv.validated, mcv)
                # But the URLs are now present
                self.assertEqual(
                    set(mcv.validated_urls.all().values_list("pk", flat=True)),
                    set(self.urls.values_list("pk", flat=True)),
                )

                # Now also make sure the timesheets of the month are all valid
                actor = UserFactory(
                    profile__affiliation_canton=canton,
                )
                QualificationFactory(
                    actor=actor,
                    session=SessionFactory(
                        day=today.replace(day=12),
                        orga=OrganizationFactory(address_canton=canton),
                    ),
                )
                # For now there is no timesheet
                initial["validated"] = True
                response = self.client.post(url, initial)
                self.assertEqual(response.status_code, 200, url)
                # The error code is a 200, as the post failed

                TimesheetFactory(
                    user=actor,
                    date=today.replace(day=12),
                    validated_at=today.replace(day=13),
                    validated_by=actor,
                )
                # Clear the cache
                timesheets_overview.timesheets_validation_status.all_users = {}
                # So post with all ticks ticked
                initial["timesheets_checked"] = True
                response = self.client.post(url, initial)
                self.assertEqual(response.status_code, 302, url)
                # The error code is a 302, as the post succeeded
                mcv = mcv_gs.first()
                self.assertTrue(mcv.validated, mcv)
                self.assertEqual(mcv.validated_by, self.client.user, mcv)
                self.assertTrue(mcv.validated_at is not None, mcv)
                # Now that it's validated, assert it can't be unvalidated
                initial["validated"] = False
                response = self.client.post(url, initial)
                self.assertEqual(response.status_code, 302, url)
                # It's still validated
                self.assertTrue(mcv_gs.first().validated)
            else:
                self.assertEqual(response.status_code, 403, url)
                # We looked at it, but it does not exist now
                self.assertFalse(mcv_gs.exists())
                # Assert that it can't be posted to either
                response = self.client.post(url, {"validated": True})
                self.assertEqual(response.status_code, 403, url)


class PowerUserTest(TestCase):
    def setUp(self):
        self.client = PowerUserAuthClient()
        super().setUp()

    def test_access_to_mcvs_lists(self):
        today = datetime.datetime.today()
        url = reverse(
            "salary:validations-month",
            kwargs={"month": today.month, "year": today.year},
        )
        # Final URL is accessible
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200, url)

        url = reverse(
            "salary:validations-year",
            kwargs={"year": today.year},
        )
        # Final URL is accessible
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200, url)

    def test_access_to_all_mcvs_updates(self):
        today = datetime.datetime.today()
        for canton in DV_STATES:
            url = reverse(
                "salary:validation-update",
                kwargs={"month": today.month, "year": today.year, "canton": canton},
            )
            # Final URL is accessible
            response = self.client.get(url, follow=True)
            self.assertEqual(response.status_code, 200, url)



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0002_auto_20150827_1722"),
    ]

    operations = [
        migrations.AlterField(
            model_name="userprofile",
            name="natel",
            field=models.CharField(max_length=13, blank=True),
        ),
    ]



# Generated by Django 2.2.12 on 2020-05-20 07:50

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0058_userprofile_fields"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="phone",
            field=models.CharField(blank=True, max_length=13, verbose_name="Téléphone"),
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations, models


def migrate_addresses(apps, schema_editor):
    # We can't import the Address model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    Address = apps.get_model("common", "Address")
    Organization = apps.get_model("orga", "Organization")
    for orga in Organization.objects.all():
        ad = Address.objects.create(
            address_street=orga.address_street,
            address_no=orga.address_no,
            address_zip=orga.address_zip,
            address_city=orga.address_city,
            address_canton=orga.address_canton,
        )
        orga.address_ptr = ad
        orga.save()


class Migration(migrations.Migration):
    dependencies = [
        ("common", "0001_initial"),
        ("orga", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="organization",
            name="address_ptr",
            field=models.ForeignKey(
                to="common.Address", null=True, on_delete=models.CASCADE
            ),
        ),
        migrations.RunPython(migrate_addresses),
        migrations.RemoveField(
            model_name="organization",
            name="id",
        ),
        migrations.AlterField(
            model_name="organization",
            name="address_ptr",
            field=models.OneToOneField(
                to="common.Address",
                auto_created=True,
                serialize=False,
                primary_key=True,
                parent_link=True,
                on_delete=models.CASCADE,
            ),
        ),
        migrations.RemoveField(
            model_name="organization",
            name="address_canton",
        ),
        migrations.RemoveField(
            model_name="organization",
            name="address_city",
        ),
        migrations.RemoveField(
            model_name="organization",
            name="address_no",
        ),
        migrations.RemoveField(
            model_name="organization",
            name="address_street",
        ),
        migrations.RemoveField(
            model_name="organization",
            name="address_zip",
        ),
    ]



# Generated by Django 3.2.14 on 2022-07-14 11:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0080_auto_20220419_1025"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicalannualstatesetting",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Configuration cantonale par année",
                "verbose_name_plural": "historical Configurations cantonales par année",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalhelperseasonworkwish",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical helper season work wish",
                "verbose_name_plural": "historical helper season work wishs",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalhelpersessionavailability",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Disponibilité par session",
                "verbose_name_plural": "historical Disponibilités par session",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalqualification",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Qualif’",
                "verbose_name_plural": "historical Qualifs",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalqualificationactivity",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Poste",
                "verbose_name_plural": "historical Postes",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalseason",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Mois",
                "verbose_name_plural": "historical Mois",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalsession",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Session",
                "verbose_name_plural": "historical Sessions",
            },
        ),
        migrations.AlterField(
            model_name="historicalannualstatesetting",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalhelperseasonworkwish",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalhelpersessionavailability",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalqualification",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalqualificationactivity",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalseason",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalsession",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("common", "0002_auto_20150914_1707"),
    ]

    operations = [
        migrations.AddField(
            model_name="address",
            name="address_additional",
            field=models.CharField(
                max_length=255, verbose_name="Complément d'adresse", blank=True
            ),
        ),
    ]



from django.db import migrations, models
from memoize import delete_memoized

from defivelo.roles import user_cantons


def ws_to_vs(apps, schema_editor):
    UserManagedState = apps.get_model("user", "UserManagedState")
    for ums in UserManagedState.objects.filter(canton="WS"):
        if UserManagedState.objects.filter(user=ums.user, canton=ums.canton).exists():
            ums.delete()
        else:
            ums.canton = "VS"
            ums.save()

    UserProfile = apps.get_model("user", "UserProfile")
    for up in UserProfile.objects.all():
        cantons = ["VS" if c == "WS" else c for c in up.activity_cantons]
        up.activity_cantons = cantons
        if up.affiliation_canton == "WS":
            up.affiliation_canton = "VS"
        up.save()
    delete_memoized(user_cantons)


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0063_use_u2019_quote"),
    ]

    operations = [
        migrations.RunPython(ws_to_vs, migrations.RunPython.noop),
        migrations.AlterField(
            model_name="usermanagedstate",
            name="canton",
            field=models.CharField(
                choices=[
                    ("AG", "Aargau"),
                    ("AR", "Appenzell Ausserrhoden"),
                    ("BS", "Basel-Stadt"),
                    ("BL", "Basel-Land"),
                    ("BE", "Berne"),
                    ("FR", "Fribourg"),
                    ("GE", "Geneva"),
                    ("GR", "Graubuenden"),
                    ("JU", "Jura"),
                    ("LU", "Lucerne"),
                    ("NE", "Neuchatel"),
                    ("SZ", "Schwyz"),
                    ("SO", "Solothurn"),
                    ("SG", "St. Gallen"),
                    ("VS", "Valais"),
                    ("VD", "Vaud"),
                    ("ZH", "Zurich"),
                ],
                max_length=2,
                verbose_name="Canton",
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="activity_cantons",
            field=models.CharField(
                blank=True,
                choices=[
                    ("AG", "Aargau"),
                    ("AR", "Appenzell Ausserrhoden"),
                    ("BS", "Basel-Stadt"),
                    ("BL", "Basel-Land"),
                    ("BE", "Berne"),
                    ("FR", "Fribourg"),
                    ("GE", "Geneva"),
                    ("GR", "Graubuenden"),
                    ("JU", "Jura"),
                    ("LU", "Lucerne"),
                    ("NE", "Neuchatel"),
                    ("SZ", "Schwyz"),
                    ("SO", "Solothurn"),
                    ("SG", "St. Gallen"),
                    ("VS", "Valais"),
                    ("VD", "Vaud"),
                    ("ZH", "Zurich"),
                ],
                max_length=50,
                verbose_name="Défi Vélo mobile",
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="affiliation_canton",
            field=models.CharField(
                blank=True,
                choices=[
                    ("", "---------"),
                    ("AG", "Aargau"),
                    ("AR", "Appenzell Ausserrhoden"),
                    ("BS", "Basel-Stadt"),
                    ("BL", "Basel-Land"),
                    ("BE", "Berne"),
                    ("FR", "Fribourg"),
                    ("GE", "Geneva"),
                    ("GR", "Graubuenden"),
                    ("JU", "Jura"),
                    ("LU", "Lucerne"),
                    ("NE", "Neuchatel"),
                    ("SZ", "Schwyz"),
                    ("SO", "Solothurn"),
                    ("SG", "St. Gallen"),
                    ("VS", "Valais"),
                    ("VD", "Vaud"),
                    ("ZH", "Zurich"),
                ],
                max_length=2,
                verbose_name="Canton d’affiliation",
            ),
        ),
    ]



# Generated by Django 2.2.16 on 2020-11-02 17:40

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0068_use_u2019_quote"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicalqualification",
            options={
                "get_latest_by": "history_date",
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Qualif’",
            },
        ),
        migrations.AlterModelOptions(
            name="qualification",
            options={
                "ordering": ["session", "created_on", "name"],
                "verbose_name": "Qualif’",
                "verbose_name_plural": "Qualifs",
            },
        ),
        migrations.AlterField(
            model_name="historicalsession",
            name="place",
            field=models.CharField(
                blank=True, max_length=512, verbose_name="Lieu de la Qualif’"
            ),
        ),
        migrations.AlterField(
            model_name="session",
            name="place",
            field=models.CharField(
                blank=True, max_length=512, verbose_name="Lieu de la Qualif’"
            ),
        ),
    ]



from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.db import migrations
from rolepermissions.checkers import has_role
from rolepermissions.permissions import grant_permission, revoke_permission


def add_user_deletions_to_power_users(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    User = get_user_model()
    for user in User.objects.all():
        if (has_role(user, "power_user")) and not user.is_superuser:
            grant_permission(user, "user_deletions")
            user.save()


def remove_user_deletions_from_power_users(apps, schema_editor):
    # We can't import the Person model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    User = get_user_model()
    for user in User.objects.all():
        if (has_role(user, "power_user")) and not user.is_superuser:
            revoke_permission(user, "user_deletions")
            user.save()


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0031_auto_20170818_1145"),
    ]

    operations = [
        migrations.RunPython(
            add_user_deletions_to_power_users, remove_user_deletions_from_power_users
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.urls import re_path
from django.views.decorators.cache import never_cache

from .views import (
    OrganizationAutocomplete,
    OrganizationCreateView,
    OrganizationDetailView,
    OrganizationListExport,
    OrganizationsListView,
    OrganizationUpdateView,
)

urlpatterns = [
    re_path(
        r"^$", never_cache(OrganizationsListView.as_view()), name="organization-list"
    ),
    re_path(
        r"^(?P<void>)?(?P<format>[a-z]+)export/$",
        never_cache(OrganizationListExport.as_view()),
        name="organization-list-export",
    ),
    re_path(r"^new/$", OrganizationCreateView.as_view(), name="organization-create"),
    re_path(
        r"^(?P<pk>[0-9]+)/$",
        never_cache(OrganizationDetailView.as_view()),
        name="organization-detail",
    ),
    re_path(
        r"^(?P<pk>[0-9]+)/update/$",
        OrganizationUpdateView.as_view(),
        name="organization-update",
    ),
    re_path(
        r"^autocomplete/$",
        OrganizationAutocomplete.as_view(),
        name="organization-autocomplete",
    ),
]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015-2020 Didier Raboud <didier.raboud@liip.ch>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django import forms
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.db.models import Q
from django.utils.translation import gettext_lazy as _

from apps.common.forms import CHPhoneNumberField
from apps.salary.models import Timesheet
from apps.user import FORMATION_KEYS, FORMATION_M2

from .. import (
    CHOSEN_AS_ACTOR,
    CHOSEN_AS_HELPER,
    CHOSEN_AS_LEADER,
    CHOSEN_AS_LEGACY,
    CHOSEN_AS_REPLACEMENT,
    MAX_MONO1_PER_QUALI,
)
from ..fields import ActorChoiceField, HelpersChoiceField, LeaderChoiceField
from ..models import Qualification
from ..models.qualification import MonitorNumberEnum, num2words


class QualificationFormQuick(forms.ModelForm):
    class Meta:
        model = Qualification
        widgets = {
            "session": forms.HiddenInput,
            "name": forms.HiddenInput,
            "class_teacher_natel": forms.HiddenInput,
        }
        fields = ["session", "name", "class_teacher_natel"]


class QualificationDeleteForm(forms.Form):
    """Validation based on POST data."""

    pass


class QualificationForm(forms.ModelForm):
    class_teacher_natel = CHPhoneNumberField(
        label=_("Natel enseignant·e"), required=False
    )

    def __init__(self, *args, **kwargs):
        session = kwargs.pop("session")
        kwargs.pop("season", None)
        kwargs.pop("cantons", None)
        is_for_coordinator = kwargs.pop("is_for_coordinator", False)
        super(QualificationForm, self).__init__(*args, **kwargs)
        other_qualifs = session.qualifications.exclude(pk=self.instance.pk)
        # Construct chosen_as dict of arrays
        chosens = {}
        for avail in session.chosen_staff.all():
            if avail.chosen_as not in chosens:
                chosens[avail.chosen_as] = []
            chosens[avail.chosen_as].append(avail.helper_id)

        legacys = chosens.get(CHOSEN_AS_LEGACY, [])
        replacements = chosens.get(CHOSEN_AS_REPLACEMENT, [])
        leaders = legacys + replacements + chosens.get(CHOSEN_AS_LEADER, [])
        helpers = legacys + replacements + chosens.get(CHOSEN_AS_HELPER, [])
        actors = legacys + replacements + chosens.get(CHOSEN_AS_ACTOR, [])

        available_staff = get_user_model().objects.exclude(
            Q(qualifs_mon2__in=other_qualifs)
            | Q(qualifs_mon1__in=other_qualifs)
            | Q(qualifs_actor__in=other_qualifs)
        )
        self.fields["leader"] = LeaderChoiceField(
            label=_("Moniteur·trice 2"),
            queryset=available_staff.filter(
                pk__in=leaders,
                profile__formation=FORMATION_M2,
            ),
            required=False,
            session=session,
        )
        self.fields["helpers"] = HelpersChoiceField(
            label=_("Moniteur·trice·s 1"),
            queryset=available_staff.filter(
                pk__in=helpers, profile__formation__in=FORMATION_KEYS
            ),
            required=False,
            session=session,
        )
        self.fields["actor"] = ActorChoiceField(
            label=_("Intervenant·e"),
            queryset=available_staff.filter(
                pk__in=actors, profile__actor_for__isnull=False
            ).distinct(),
            required=False,
            session=session,
        )
        self.fields["n_helpers"].required = False
        if is_for_coordinator:
            # For non-stateManagers (coordinator), delete non-accessible fields
            for f in list(self.fields.keys()):
                if f not in [
                    "session",
                    "name",
                    "class_teacher_fullname",
                    "class_teacher_natel",
                    "n_participants",
                    "n_bikes",
                    "n_helmets",
                    "n_helpers",
                ]:
                    del self.fields[f]

    def clean_helpers(self):
        # Check that we don't have too many moniteurs 1
        helpers = self.cleaned_data.get("helpers")
        max_m1 = (
            MonitorNumberEnum(self.cleaned_data.get("n_helpers")).m1
            if self.cleaned_data.get("n_helpers")
            else MAX_MONO1_PER_QUALI
        )
        if helpers and helpers.count() > max_m1:
            raise ValidationError(
                _("Pas plus de %s moniteur·trices 1 !") % num2words(max_m1)
            )
        # Check that all moniteurs are unique
        all_leaders_pk = []
        leader = self.cleaned_data.get("leader")
        if leader:
            all_leaders_pk.append(leader.pk)
        if helpers:
            for helper in helpers.all():
                all_leaders_pk.append(helper.pk)
        # Check unicity
        seen_pk = set()
        if len(
            [x for x in all_leaders_pk if x not in seen_pk and not seen_pk.add(x)]
        ) < len(all_leaders_pk):
            raise ValidationError(
                _("Il y a des moniteur·trice·s à double !"), code="double-helpers"
            )
        return helpers

    def clean_actor(self):
        # Check that the picked actor corresponds to the activity_C
        actor = self.cleaned_data.get("actor")
        activity_C = self.cleaned_data.get("activity_C")
        if actor:
            if activity_C:
                if not actor.profile.actor_for.filter(id=activity_C.id).exists():
                    raise ValidationError(
                        _(
                            "L’intervenant·e n’est pas qualifié·e pour la rencontre "
                            "prévue !"
                        ),
                        code="unqualified-actor",
                    )
            helpers = self.cleaned_data.get("helpers")
            if helpers and helpers.filter(id=actor.id).exists():
                raise ValidationError(
                    _("L’intervenant·e ne peut pas aussi être moniteur·trice !"),
                    code="helper-actor",
                )
        return actor

    def clean(self):
        # Check that there are <= helmets or bikes than participants
        n_participants = self.cleaned_data.get("n_participants")
        if not n_participants:
            n_participants = 0
        n_bikes = self.cleaned_data.get("n_bikes")
        if n_bikes and int(n_bikes) > int(n_participants):
            raise ValidationError(
                _("Il y a trop de vélos prévus !"), code="too-many-bikes"
            )
        n_helmets = self.cleaned_data.get("n_helmets")
        if n_helmets and int(n_helmets) > int(n_participants):
            raise ValidationError(
                _("Il y a trop de casques prévus !"), code="too-many-helmets"
            )

        if self.instance.pk:
            removed_helpers = {user.pk for user in self.instance.helpers.all()} - {
                user.pk for user in self.cleaned_data.get("helpers", [])
            }
            if removed_helpers:
                Timesheet.objects.filter(
                    user__in=removed_helpers, date=self.instance.session.day
                ).delete()

            if self.instance.leader != self.cleaned_data.get("leader"):
                Timesheet.objects.filter(
                    user=self.instance.leader, date=self.instance.session.day
                ).delete()

            if self.instance.actor != self.cleaned_data.get("actor"):
                Timesheet.objects.filter(
                    user=self.instance.actor, date=self.instance.session.day
                ).delete()

        return self.cleaned_data

    class Meta:
        model = Qualification
        widgets = {"session": forms.HiddenInput}
        fields = [
            "session",
            "name",
            "class_teacher_fullname",
            "class_teacher_natel",
            "n_participants",
            "n_bikes",
            "n_helmets",
            "n_helpers",
            "leader",
            "helpers",
            "activity_A",
            "activity_B",
            "activity_C",
            "actor",
            "comments",
        ]



from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0027_auto_20161010_1310"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="qualification",
            name="route",
        ),
    ]



# Generated by Django 1.11.4 on 2017-09-05 13:12
from __future__ import unicode_literals

from django.db import migrations, models


def chosen_to_chosen_as(apps, schema_editor):
    # We can't import the HelperSessionAvailability model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    HelperSessionAvailability = apps.get_model("challenge", "HelperSessionAvailability")
    for hsa in HelperSessionAvailability.objects.all():
        hsa.chosen_as = 1 if hsa.chosen else 0
        hsa.save()


def chosen_as_to_chosen(apps, schema_editor):
    # We can't import the HelperSessionAvailability model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    HelperSessionAvailability = apps.get_model("challenge", "HelperSessionAvailability")
    for hsa in HelperSessionAvailability.objects.all():
        hsa.chosen = hsa.chosen_as != 0
        hsa.save()


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0045_auto_20170904_1750"),
    ]

    operations = [
        migrations.AddField(
            model_name="helpersessionavailability",
            name="chosen_as",
            field=models.PositiveSmallIntegerField(
                choices=[
                    (0, "Pas choisi"),
                    (1, "Choisi"),
                    (2, "Comme intervenant"),
                    (3, "Moniteur 1"),
                    (4, "Moniteur 2"),
                ],
                default=0,
                verbose_name="S\xe9lectionn\xe9 pour la session comme",
            ),
        ),
        migrations.RunPython(chosen_to_chosen_as, chosen_as_to_chosen),
    ]



# Generated by Django 4.2.6 on 2025-12-09 10:15

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('challenge', '0087_translate_qualification_activity'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicalqualification',
            name='n_participants',
            field=models.PositiveSmallIntegerField(blank=True, null=True, validators=[django.core.validators.MaxValueValidator(30)], verbose_name='Nombre de participant·es'),
        ),
        migrations.AlterField(
            model_name='qualification',
            name='n_participants',
            field=models.PositiveSmallIntegerField(blank=True, null=True, validators=[django.core.validators.MaxValueValidator(30)], verbose_name='Nombre de participant·es'),
        ),
    ]



import queue
import threading

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.mail import send_mail
from django.core.signals import request_finished
from django.db.models.signals import post_save, pre_save
from django.dispatch import receiver
from django.utils.translation import gettext_lazy as _

from apps.user.models import UserProfile

# Global dictionary to store field changes temporarily during the save process
_user_changes = {}  # Payload indexed by user.pk
_user_changes_lock = threading.Lock()
_userprofile_to_notify = queue.Queue()  # User model that are used in user_changes.


@receiver(pre_save, sender=get_user_model())
def user_email_change_signal(sender, instance, **kwargs):
    """
    Signal to detect email changes on the User model
    Compute _user_changes if the email changes
    """
    if not instance.pk:
        return
    try:
        old_instance = sender.objects.get(pk=instance.pk)
        if old_instance.email != instance.email:
            # Store email change in global dictionary
            user_id = instance.pk
            with _user_changes_lock:
                _user_changes.setdefault(user_id, []).append(
                    {
                        "field": "email",
                        "old_value": old_instance.email,
                        "new_value": instance.email,
                    }
                )
    except sender.DoesNotExist:
        pass


@receiver(pre_save, sender=UserProfile)
def userprofile_field_change_signal(sender, instance, **kwargs):
    """
    Signal to detect changes on specific fields of the UserProfile model
    Compute _user_changes if the fields changes.
    """

    def normalize_iban_value(value):
        """The display format for IBAN has a space every 4 characters."""
        if value is None:
            return value
        grouping = 4
        value = value.upper().replace(" ", "").replace("-", "")
        return " ".join(value[i : i + grouping] for i in range(0, len(value), grouping))

    if not instance.pk:
        return
    try:
        old_instance = sender.objects.get(pk=instance.pk)
        changes = []

        # Check IBAN changes,
        # - oldvalue is stored without space, and new one with some space, so we normalize the value
        old_iban = normalize_iban_value(old_instance.iban)
        new_iban = normalize_iban_value(instance.iban)
        if old_iban != new_iban and str(old_iban).strip():
            changes.append(
                {
                    "field": "IBAN",
                    "old_value": old_iban,
                    "new_value": new_iban,
                }
            )

        # Check address field changes
        address_fields = [
            "address_street",
            "address_no",
            "address_additional",
            "address_zip",
            "address_city",
            "address_canton",
        ]
        for field in address_fields:
            old_value = getattr(old_instance, field)
            new_value = getattr(instance, field)
            if old_value != new_value and str(old_value).strip():
                changes.append(
                    {"field": field, "old_value": old_value, "new_value": new_value}
                )

        # Store changes in global dictionary
        if changes:
            user_id = instance.user.pk
            with _user_changes_lock:
                _user_changes.setdefault(user_id, []).extend(changes)

    except sender.DoesNotExist:
        pass


@receiver(post_save, sender=get_user_model())
@receiver(post_save, sender=UserProfile)
def userprofile_mark_save_notification(sender, instance, **kwargs):
    """
    On post save, mark the user to be processed with changes given there is a payload with change for this user.
    """
    user = instance if sender == get_user_model() else instance.user

    if user:
        try:
            with _user_changes_lock:
                if user.pk in _user_changes:
                    _userprofile_to_notify.put_nowait(user)
        except queue.Full:
            pass


@receiver(request_finished)
def do_userprofile_notification(**kwargs):
    """
    On request finished, we regroup all changes by user and send an email.
    We loop on _userprofile_to_notify to know the user and on _user_changes for the payload.
    """
    while not _userprofile_to_notify.empty():
        try:
            user = _userprofile_to_notify.get_nowait()
            with _user_changes_lock:
                changes = _user_changes.pop(user.pk, {})
            _send_field_change_notification(user, changes)
        except queue.Empty:
            break


def _send_field_change_notification(user, changes):
    """
    Send an email notification with all field changes for a user
    """
    if not changes or not settings.PROFILE_CHANGED_NOTIFY_EMAIL:
        return

    # Prepare email content
    subject = _("Notification de modification de données utilisateur·trice")

    # Build the email body
    body_lines = [
        _("Bonjour,"),
        "",
        _(
            "Les informations suivantes pour l’utilisateur·trice {user} / {user_id} ont été modifiées:"
        ).format(user=user.get_full_name(), user_id=user.pk),
        "",
        "",
    ]

    for change in changes:
        field_name = change["field"]
        old_val = change["old_value"] or _("(vide)")
        new_val = change["new_value"] or _("(vide)")
        body_lines.append(f"• {field_name}: {old_val} → {new_val}")

    body_lines.extend(
        [
            "",
            _("Ceci est un message automatique."),
            _("Merci de mettre à jour la base de données des salaires en conséquences"),
        ]
    )

    body = "\n".join([str(b) for b in body_lines])

    # Send email to administrators or relevant recipients
    try:
        send_mail(
            subject,
            body,
            settings.DEFAULT_FROM_EMAIL,
            list(filter(None, settings.PROFILE_CHANGED_NOTIFY_EMAIL.split(","))),
            fail_silently=False,
        )
    except Exception as e:
        print(f"Failed to send field change notification email: {e}")



# Generated by Django 1.11.4 on 2017-08-22 08:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0034_auto_20170821_1702"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="formation_firstdate",
            field=models.DateField(
                blank=True, null=True, verbose_name="Date de la première formation"
            ),
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0003_session_organization"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="session",
            unique_together=set([("organization", "timeslot", "day")]),
        ),
    ]



import functools
import inspect
import os
import random
import subprocess
import time
from datetime import datetime
from io import StringIO
from tempfile import NamedTemporaryFile

import dj_database_url
from dulwich import porcelain
from fabric import task
from fabric.connection import Connection
from invoke import Exit
from invoke.context import Context
from invoke.exceptions import UnexpectedExit

ENVIRONMENTS = {
    "prod": {
        "root": "/var/www/intranet.defi-velo.ch/prod/",
        # "host": "wpy10809@onhp-python3.iron.bsa.oriented.ch:29992",  # OLD
        "host": "wpy10809@onhp-python2.iron.bsa.oriented.ch:29992",  # NEW
        "pid": "/run/uwsgi/app/intranet.defi-velo.ch/pid",
        "ini": "/etc/uwsgi/apps-enabled/intranet.defi-velo.ch.ini",
        "settings": {
            "ALLOWED_HOSTS": "\n".join(["intranet.defi-velo.ch"]),
            "MEDIA_URL": "/media/",
            "STATIC_URL": "/static/",
            "MEDIA_ROOT": "/var/www/intranet.defi-velo.ch/prod/media/",
            "STATIC_ROOT": "/var/www/intranet.defi-velo.ch/prod/static/",
            "SITE_DOMAIN": "intranet.defi-velo.ch",
            "SASSC_BIN_PATH": "/var/www/intranet.defi-velo.ch/prod/venv/bin/pysassc",
            "VIRTUAL_ENV": "/var/www/intranet.defi-velo.ch/prod/venv",
        },
    },
    "staging": {
        "root": "/var/www/intranet.defi-velo.ch/staging/",
        # "host": "wpy10809@onhp-python3.iron.bsa.oriented.ch:29992",  # OLD
        "host": "wpy10809@onhp-python2.iron.bsa.oriented.ch:29992",  # NEW
        "pid": "/run/uwsgi/app/staging.intranet.defi-velo.ch/pid",
        "ini": "/etc/uwsgi/apps-enabled/staging.intranet.defi-velo.ch.ini",
        "requirements": "staging",
        "settings": {
            "ALLOWED_HOSTS": "\n".join(["staging.intranet.defi-velo.ch"]),
            "MEDIA_URL": "/media/",
            "STATIC_URL": "/static/",
            "MEDIA_ROOT": "/var/www/intranet.defi-velo.ch/staging/media/",
            "STATIC_ROOT": "/var/www/intranet.defi-velo.ch/staging/static/",
            "SITE_DOMAIN": "staging.intranet.defi-velo.ch",
            "SASSC_BIN_PATH": "/var/www/intranet.defi-velo.ch/staging/venv/bin/pysassc",
            "VIRTUAL_ENV": "/var/www/intranet.defi-velo.ch/staging/venv",
            "EMAIL_URL": "file:////tmp/dv_emails",
            "DJANGO_SETTINGS_MODULE": "defivelo.settings.staging",
        },
    },
}

project_name = "defivelo"
project_name_verbose = "Intranet Défi Vélo"

PRODUCTION_DB_NAME = "intranetdefiveloch001"


def remote(task_func):
    """
    Decorate task functions to check for presence of a Connection instance in their context.
    Also pass the Connection instance in argument for convenience.
    """

    @functools.wraps(task_func)
    def call_task_with_connection(ctx, *args, **kwargs):
        if not hasattr(ctx, "conn"):
            raise RuntimeError("Trying to run a remote task with no environment loaded")
        return task_func(ctx, *args, **kwargs)

    call_task_with_connection.__signature__ = inspect.signature(task_func)
    return call_task_with_connection


def ensure_absolute_path(path):
    if not os.path.isabs(path):
        raise ValueError("{!r} is not an absolute path.".format(path))


class CustomConnection(Connection):
    """
    Add helpers function on Connection
    """

    @property
    def site_root(self):
        return self.config.root

    @property
    def project_root(self):
        """
        Return the path to the root of the project on the remote server.
        """
        return os.path.join(self.site_root, project_name)

    @property
    def venv_path(self):
        return os.path.join(self.site_root, "venv")

    @property
    def envdir_path(self):
        return os.path.join(self.project_root, "envdir")

    @property
    def backups_root(self):
        """
        Return the path to the backups directory on the remote server.
        """
        return os.path.join(self.site_root, "backups")

    @property
    def media_root(self):
        """
        Return the path to the media directory on the remote server.
        """
        try:
            path = self.config.settings["MEDIA_ROOT"]
        except KeyError:
            return os.path.join(self.site_root, "media")
        else:
            ensure_absolute_path(path)
            return path

    @property
    def static_root(self):
        """
        Return the path to the static directory on the remote server.
        """
        try:
            path = self.config.settings["STATIC_ROOT"]
        except KeyError:
            return os.path.join(self.site_root, "static")
        else:
            ensure_absolute_path(path)
            return path

    def run_in_project_root(self, cmd, **kwargs):
        """
        Run command after a cd to the project_root
        """
        with self.cd(self.project_root):
            return self.run(cmd, **kwargs)

    def git(self, gitcmd, **kwargs):
        """
        git from the project_root
        """
        return self.run_in_project_root("git {}".format(gitcmd), **kwargs)

    def run_in_venv(self, cmd, args, **run_kwargs):
        """
        Binaries from the venv
        """
        return self.run_in_project_root(
            "{} {}".format(os.path.join(self.venv_path, "bin", cmd), args), **run_kwargs
        )

    def mk_venv(self, **run_kwargs):
        """
        Create the venv
        """

        with self.cd(self.site_root):
            self.run("python3 -m venv venv", **run_kwargs)

    def pip(self, args, **run_kwargs):
        """
        pip from the venv, in the project_root
        """
        return self.run_in_venv("pip", args, **run_kwargs)

    def python(self, args, **run_kwargs):
        """
        python from the venv, in the project_root
        """
        return self.run_in_venv("python", args, **run_kwargs)

    def manage_py(self, args, debug=False, **run_kwargs):
        """
        manage.py with the python from the venv, in the project_root
        """
        try:
            env = {
                "DJANGO_SETTINGS_MODULE": self.config.settings["DJANGO_SETTINGS_MODULE"]
            }
        except KeyError:
            env = {}
        if debug:
            env["DEBUG"] = 1
        return self.python("./manage.py {}".format(args), env=env, **run_kwargs)

    def set_setting(self, name, value=None, force: bool = True):
        """
        Set a setting in the environment directory, for use by Django
        """
        envfile_path = os.path.join(self.envdir_path, name)

        will_write = force
        if not force:
            try:
                # Test that it does exist
                self.run_in_project_root("test -r {}".format(envfile_path), hide=True)
            except UnexpectedExit:
                will_write = True

        if will_write:
            if value is None:
                value = input("Value for {}: ".format(name))

            # Convert booleans into values understood as such by Django
            if isinstance(value, bool):
                value = "1" if value else ""
            self.put(StringIO("{}\n".format(value)), envfile_path)

    def db_creds(self):
        """
        Return DB dictionary of credentials from the DATABASE_URL envdir var
        """
        with self.cd(self.project_root):
            db_credentials = self.run(
                "cat envdir/DATABASE_URL", hide=True
            ).stdout.strip()
        db_credentials_dict = dj_database_url.parse(db_credentials)

        if not is_supported_db_engine(db_credentials_dict["ENGINE"]):
            raise NotImplementedError(
                "The dump_db task doesn't support the remote database engine"
            )
        return db_credentials_dict

    def dump_db(self, destination):
        """
        Dump the database to the given directory and return the path to the file created.
        This creates a gzipped SQL file.
        """
        db_credentials_dict = self.db_creds()

        outfile = os.path.join(
            destination, datetime.now().strftime("%Y-%m-%d_%H%M%S.sql.gz")
        )

        self.run(
            "pg_dump -O -x -h '{host}' -U '{user}' '{db}'|gzip > {outfile}".format(
                host=db_credentials_dict["HOST"],
                user=db_credentials_dict["USER"],
                db=db_credentials_dict["NAME"],
                outfile=outfile,
            ),
            env={"PGPASSWORD": db_credentials_dict["PASSWORD"].replace("$", "\$")},
        )

        return outfile

    def drop_and_recreate_db(
        self, production_db_name: str = PRODUCTION_DB_NAME, **kwargs
    ):
        """
        Drop and recreate an empty DB for this environment
        """
        db_creds = self.db_creds()
        try:
            confirmed = kwargs.pop("confirm_that_this_is_really_what_I_want")
        except KeyError:
            confirmed = False

        if not confirmed:
            raise Exit("You need to pass the righteous argument to proceed with this")

        if db_creds["NAME"] == production_db_name:
            raise Exit("This is the production DB, abort.")
        self.run(
            f"dropdb -h '{db_creds['HOST']}' -U '{db_creds['USER']}' '{db_creds['NAME']}'",
            env={"PGPASSWORD": db_creds["PASSWORD"].replace("$", "\$")},
        )
        self.run(
            f"createdb -h '{db_creds['HOST']}' -U '{db_creds['USER']}' '{db_creds['NAME']}'",
            env={"PGPASSWORD": db_creds["PASSWORD"].replace("$", "\$")},
        )

    def restore_dump(self, db_dump_gz):
        """
        Restore a gz DB dump to our database.
        """
        db_creds = self.db_creds()
        cmd = (
            f"gunzip -c {db_dump_gz} "
            f"| sed -E '/\\\\unrestrict/d' "
            f"| psql -h '{db_creds['HOST']}' -U '{db_creds['USER']}' '{db_creds['NAME']}'",
        )
        self.run(cmd, env={"PGPASSWORD": db_creds["PASSWORD"].replace("$", r"\$")})

    def create_structure(self):
        """
        Create the basic directory structure on the remote server.
        """
        command = " ".join(
            [
                "mkdir -p",
                self.project_root,
                self.backups_root,
                self.static_root,
                self.media_root,
            ]
        )
        self.run(command)

    def clean_old_database_backups(self, nb_backups_to_keep):
        """
        Remove old database backups from the system and keep `nb_backups_to_keep`.
        """
        backups = self.ls(self.backups_root)
        backups = sorted(backups, reverse=True)

        if len(backups) > nb_backups_to_keep:
            backups_to_delete = backups[nb_backups_to_keep:]
            file_to_remove = [
                os.path.join(self.backups_root, backup_to_delete)
                for backup_to_delete in backups_to_delete
            ]
            self.run('rm "%s"' % '" "'.join(file_to_remove))
            print("%d backups deleted." % len(backups_to_delete))
        else:
            print("No backups to delete.")

    def ls(self, path):
        """
        Return the list of the files in the given directory, omitting . and ...
        """
        with self.cd(path):
            files = self.run("for i in *; do echo $i; done", hide=True).stdout.strip()
            files_list = files.replace("\r", "").split("\n")

        return files_list


def generate_secret_key():
    """
    Generate a random secret key, suitable to be used as a SECRET_KEY setting.
    """
    return "".join(
        [
            random.SystemRandom().choice(
                "abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)"
            )
            for i in range(50)
        ]
    )


def is_supported_db_engine(engine):
    return engine in [
        "django.db.backends.postgresql_psycopg2",
        "django.db.backends.postgresql",
    ]


@task
@remote
def fetch_db(c, destination="."):
    """
    Dump the database on the remote host and retrieve it locally.

    The destination parameter controls where the dump should be stored locally.
    """
    dump_path = c.conn.dump_db("~")
    filename = os.path.basename(dump_path)

    subprocess.run(
        [
            "scp",
            "-P",
            str(c.conn.port),
            "{user}@{host}:{directory}".format(
                user=c.conn.user, host=c.conn.host, directory=dump_path
            ),
            destination,
        ]
    )
    c.conn.run("rm %s" % dump_path)

    return os.path.join(destination, filename)


@task
@remote
def reset_db_from_prod(c):
    """
    Restore the given database dump.

    The dump must be a gzipped SQL dump. If the dump_file parameter is not set,
    the database will be dumped and retrieved from the remote host.
    """
    if not c.environment == "staging":
        raise Exit(
            "reset_db_from_prod can only be executed for the staging environment"
        )

    # Get PROD connection information
    try:
        prod_conf = ENVIRONMENTS["prod"]
    except KeyError:
        raise Exit("reset_db_from_prod needs to access the 'prod' environment")

    #  Do a pre-backup
    c.conn.dump_db(c.conn.backups_root)

    prod_ctx = Context(prod_conf)
    prod_ctx.conn = CustomConnection(host=prod_conf["host"], inline_ssh_env=True)
    prod_ctx.conn.config.load_overrides(prod_conf)

    with NamedTemporaryFile() as local_transit_file:
        # Handle the production part:
        with prod_ctx.conn as conn:
            # Dump the PROD DB
            prod_dump_path = conn.dump_db("~")

            # Download it here, assuming PROD and STAGING can be on complete different servers (it's not the case I know)
            subprocess.run(
                [
                    "scp",
                    "-P",
                    str(conn.port),
                    f"{conn.user}@{conn.host}:{prod_dump_path}",
                    local_transit_file.name,
                ]
            )

            # Delete the PROD dump from the server
            conn.run("rm %s" % prod_dump_path)

        # Now push to staging
        with c.conn as conn:
            # Put it to ~
            subprocess.run(
                [
                    "scp",
                    "-P",
                    str(conn.port),
                    local_transit_file.name,
                    f"{conn.user}@{conn.host}:{prod_dump_path}",
                ]
            )
            # OK. Now we have a prod backup on the staging environment. Proceed to restoring it
            conn.drop_and_recreate_db(confirm_that_this_is_really_what_I_want=True)
            # Restore the dump there
            conn.restore_dump(prod_dump_path)
            # Now remove the dump from the remote path too
            conn.run("rm %s" % prod_dump_path)

    # Run what's needed to bring the target env to a working state
    dj_migrate_database(c)
    dj_sync_roles(c)
    c.conn.manage_py(
        f'set_default_site --name "{project_name_verbose} ({c.environment})" --domain "{c.config.settings["SITE_DOMAIN"]}"'
    )
    c.conn.manage_py("set_fake_passwords", debug=True)
    restart_uwsgi(c)


@task
def import_db(c, dump_file=None):
    """
    Restore the given database dump.

    The dump must be a gzipped SQL dump. If the dump_file parameter is not set,
    the database will be dumped and retrieved from the remote host.
    """

    if not dump_file and not hasattr(c, "conn"):
        raise RuntimeError("Must either specify a dump_file or an environment")

    db_credentials = os.environ.get("DATABASE_URL")
    if not db_credentials:
        with open("envdir/DATABASE_URL", "r") as db_credentials_file:
            db_credentials = db_credentials_file.read()
    db_credentials_dict = dj_database_url.parse(db_credentials)

    if not is_supported_db_engine(db_credentials_dict["ENGINE"]):
        raise NotImplementedError(
            "The import_db task doesn't support your database engine"
        )

    if dump_file is None:
        dump_file = fetch_db(c)

    db_info = {
        "host": db_credentials_dict["HOST"],
        "user": db_credentials_dict["USER"],
        "db": db_credentials_dict["NAME"],
        "db_dump": dump_file,
    }
    env = {"PGPASSWORD": db_credentials_dict["PASSWORD"].replace("$", "\$")}

    c.run(
        """
        psql -h '{host}' -U '{user}' '{db}' '$(./manage.py sqldsn -q)' <<<'
            SELECT pg_terminate_backend(pid)
            FROM pg_stat_activity
            WHERE pid != pg_backend_pid()'
        """.format(**db_info),
        env=env,
    )

    c.run("dropdb -h '{host}' -U '{user}' '{db}'".format(**db_info), env=env)
    c.run("createdb -h '{host}' -U '{user}' '{db}'".format(**db_info), env=env)
    c.run(
        "gunzip -c {db_dump}| sed -E '/\\\\unrestrict/d' | psql -h '{host}' -U '{user}' '{db}'".format(
            **db_info
        ),
        env=env,
    )


@task
@remote
def deploy(c):
    """
    Execute all deployment steps
    """
    c.conn.create_structure()
    push_code_update(c, "HEAD")
    sync_settings(c)
    c.conn.dump_db(c.conn.backups_root)
    install_requirements(c)
    dj_collect_static(c)
    dj_migrate_database(c)
    dj_sync_roles(c)
    compile_messages(c)
    restart_uwsgi(c)
    c.conn.clean_old_database_backups(nb_backups_to_keep=10)


@task
@remote
def push_code_update(c, git_ref):
    """
    Synchronize the remote code repository
    """
    with c.conn.cd(c.conn.project_root):
        # First, check that the remote deployment directory exists
        try:
            c.conn.run("test -d .", hide=True)
        except UnexpectedExit:
            raise Exit(
                "Provisioning not finished, directory {} doesn't exist!".format(
                    c.config["root"]
                )
            )
        # Now make sure there's git, and a git repository
        try:
            c.conn.git("--version", hide=True)
        except UnexpectedExit:
            raise Exit("Provisioning not finished, git not available!")

        try:
            c.conn.git("rev-parse --git-dir", hide=True)
        except UnexpectedExit:
            c.conn.git("init")

    git_remote_url = "ssh://{user}@{host}:{port}/{directory}".format(
        user=c.conn.user,
        host=c.conn.host,
        port=c.conn.port,
        directory=c.conn.project_root,
    )

    # Now push our code to the remote, always as FABHEAD branch
    porcelain.push(".", git_remote_url, "{}:FABHEAD".format(git_ref))

    with c.conn.cd(c.conn.project_root):
        c.conn.git("checkout -f -B master FABHEAD", hide=True)
        c.conn.git("branch -d FABHEAD", hide=True)
        # Force using an absolute URL for locale submodule since the relative one would fail
        # (this uses a deploy key generated on the server)
        c.conn.git(
            "config submodule.locale.url git@gitlab.liip.ch:swing/defivelo/intranet-i18n.git",
            hide=True,
        )
        c.conn.git("submodule update --init", hide=True)

        vcs_commit = subprocess.run(
            ["git", "rev-parse", git_ref], stdout=subprocess.PIPE
        ).stdout.decode()
        vcs_version = subprocess.run(
            ["git", "describe", "--tags", git_ref], stdout=subprocess.PIPE
        ).stdout.decode()
        c.conn.set_setting("VCS_COMMIT", vcs_commit)
        c.conn.set_setting("VCS_VERSION", vcs_version)


@task
@remote
def install_requirements(c):
    """
    Install project requirements in venv
    """
    try:
        c.conn.run("test -r {}".format(c.conn.venv_path), hide=True)
    except UnexpectedExit:
        c.conn.mk_venv()

    try:
        requirements_variant = c.config.requirements
    except AttributeError:
        requirements_variant = "base"

    c.conn.pip(f"install -r requirements/{requirements_variant}.txt")


@task
@remote
def sync_settings(c):
    """
    Synchronize the settings from the above environment to the server
    """

    required_settings = set(
        [
            "DATABASE_URL",
            "MEDIA_ROOT",
            "STATIC_ROOT",
            "MEDIA_URL",
            "STATIC_URL",
            "ALLOWED_HOSTS",
        ]
    )

    env_settings = getattr(c.config, "settings", {})
    for setting, value in env_settings.items():
        c.conn.set_setting(setting, value=value)

    # Ask for settings that are required but were not set in the parameters
    # file
    for setting in required_settings - set(env_settings.keys()):
        c.conn.set_setting(setting, force=False)

    c.conn.set_setting(
        "DJANGO_SETTINGS_MODULE",
        value="%s.settings.base" % project_name,
        force=False,
    )
    c.conn.set_setting("SECRET_KEY", value=generate_secret_key(), force=False)


@task
@remote
def dj_collect_static(c):
    """
    Django: collect the statics
    """
    c.conn.manage_py("collectstatic --noinput")


@task
@remote
def dj_migrate_database(c):
    """
    Django: Migrate the database
    """
    c.conn.manage_py("migrate")


@task
@remote
def dj_sync_roles(c):
    """
    Django: Synchronize Groups/Permissions with defivelo.roles
    """
    c.conn.manage_py("sync_roles --reset_user_permissions")


@task
@remote
def restart_uwsgi(c):
    """
    Restart uWSGI processes
    """
    c.conn.run(f"uwsgi --stop '{c.conn.config.pid}'")
    time.sleep(2)
    c.conn.run(f"uwsgi --start '{c.conn.config.ini}'")


@task
@remote
def compile_messages(c):
    c.conn.manage_py("compilemessages -l fr -l de -l en -l it")


# Environment handling stuff
############################
def create_environment_task(name, env_conf):
    """
    Create a task function from an environment name
    """

    @task(name=name)
    def load_environment(ctx):
        conf = env_conf.copy()
        conf["environment"] = name
        # So now conf is the ENVIRONMENTS[env] dict plus "environment" pointing to the name
        # Push them in the context config dict
        ctx.config.load_overrides(conf)
        # Add the common_settings in there
        ctx.conn = CustomConnection(host=conf["host"], inline_ssh_env=True)
        ctx.conn.config.load_overrides(conf)

    load_environment.__doc__ = (
        """Prepare connection and load config for %s environment""" % name
    )
    return load_environment


def load_environments_tasks(environments):
    """
    Load environments as fabric tasks
    """
    for name, env_conf in environments.items():
        globals()[name] = create_environment_task(name, env_conf)


# Yes, do it
load_environments_tasks(ENVIRONMENTS)



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0005_auto_20150831_1624"),
    ]

    operations = [
        migrations.CreateModel(
            name="SessionActivity",
            fields=[
                (
                    "id",
                    models.AutoField(
                        serialize=False,
                        primary_key=True,
                        verbose_name="ID",
                        auto_created=True,
                    ),
                ),
                (
                    "category",
                    models.CharField(
                        max_length=1,
                        verbose_name="Catégorie",
                        blank=True,
                        choices=[
                            ("A", "Vélo dans la rue"),
                            ("B", "Mécanique"),
                            ("C", "Rencontre"),
                        ],
                    ),
                ),
            ],
            options={
                "verbose_name": "Poste",
                "verbose_name_plural": "Postes",
                "ordering": ["category", "pk"],
            },
        ),
        migrations.CreateModel(
            name="SessionActivityTranslation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        serialize=False,
                        primary_key=True,
                        verbose_name="ID",
                        auto_created=True,
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        max_length=15, verbose_name="Language", db_index=True
                    ),
                ),
                ("name", models.CharField(max_length=255, verbose_name="Nom")),
                (
                    "master",
                    models.ForeignKey(
                        to="challenge.SessionActivity",
                        editable=False,
                        related_name="translations",
                        null=True,
                        on_delete=models.CASCADE,
                    ),
                ),
            ],
            options={
                "db_table": "challenge_sessionactivity_translation",
                "managed": True,
                "db_tablespace": "",
                "verbose_name": "Poste Translation",
                "default_permissions": (),
            },
        ),
        migrations.AddField(
            model_name="qualification",
            name="activity_A",
            field=models.ForeignKey(
                to="challenge.SessionActivity",
                related_name="sessions_A",
                blank=True,
                null=True,
                verbose_name="Vélo dans la rue",
                on_delete=models.CASCADE,
            ),
        ),
        migrations.AddField(
            model_name="qualification",
            name="activity_B",
            field=models.ForeignKey(
                to="challenge.SessionActivity",
                related_name="sessions_B",
                blank=True,
                null=True,
                verbose_name="Mécanique",
                on_delete=models.CASCADE,
            ),
        ),
        migrations.AddField(
            model_name="qualification",
            name="activity_C",
            field=models.ForeignKey(
                to="challenge.SessionActivity",
                related_name="sessions_C",
                blank=True,
                null=True,
                verbose_name="Rencontre",
                on_delete=models.CASCADE,
            ),
        ),
        migrations.AlterUniqueTogether(
            name="sessionactivitytranslation",
            unique_together=set([("language_code", "master")]),
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.test import TestCase
from django.urls import reverse
from django.utils import timezone

from apps.article.models import Article
from defivelo.tests.utils import AuthClient, CoordinatorAuthClient, PowerUserAuthClient


class OneArticle(object):
    def setUp(self):
        self.article = Article.objects.create(
            title="TestArticle",
            body="<h1>Test Body</h1>",
            published=True,
            modified=timezone.now(),
        )


class AuthUserTest(OneArticle, TestCase):
    def setUp(self):
        super().setUp()
        self.client = AuthClient()

    def test_my_restrictions(self):
        for symbolicurl in [
            "article-update",
            "article-delete",
        ]:
            response = self.client.get(
                reverse(symbolicurl, kwargs={"pk": self.article.pk})
            )
            self.assertEqual(response.status_code, 403, symbolicurl)

        response = self.client.get(reverse("article-create"))
        self.assertEqual(response.status_code, 403, "Article creation")

    def test_articles_are_accessible(self):
        response = self.client.get(reverse("home"))
        self.assertEqual(response.status_code, 200, "home")
        self.assertContains(response, "TestArticle")


class CoordinatorTest(OneArticle, TestCase):
    def setUp(self):
        super().setUp()
        self.client = CoordinatorAuthClient()

    def test_articles_are_not_accessible(self):
        response = self.client.get(reverse("home"))
        self.assertEqual(response.status_code, 200, "home")
        # The only article is not present
        self.assertNotContains(response, "TestArticle")


class PowerUserTest(OneArticle, TestCase):
    def setUp(self):
        super().setUp()
        self.client = PowerUserAuthClient()

    def test_my_accesses(self):
        for symbolicurl in [
            "article-update",
            "article-delete",
        ]:
            response = self.client.get(
                reverse(symbolicurl, kwargs={"pk": self.article.pk})
            )
            self.assertEqual(response.status_code, 200, symbolicurl)

        response = self.client.get(reverse("article-create"))
        self.assertEqual(response.status_code, 200, "Article creation request")

        response = self.client.post(
            reverse("article-create"),
            {"title": "New article", "body": "New body", "published": False},
        )
        self.assertEqual(response.status_code, 302, "Article creation")



from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("challenge", "0008_qualification_helpers"),
    ]

    operations = [
        migrations.AddField(
            model_name="qualification",
            name="actor",
            field=models.ForeignKey(
                verbose_name="Intervenant",
                blank=True,
                null=True,
                to=settings.AUTH_USER_MODEL,
                related_name="sessions_actor",
                on_delete=models.CASCADE,
            ),
        ),
    ]



from __future__ import unicode_literals

import datetime

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0014_auto_20150907_1143"),
    ]

    operations = [
        migrations.AddField(
            model_name="session",
            name="begin",
            field=models.TimeField(null=True, verbose_name="Début", blank=True),
        ),
        migrations.AddField(
            model_name="session",
            name="duration",
            field=models.DurationField(
                verbose_name="Durée", default=datetime.timedelta(0, 10800)
            ),
        ),
        migrations.AlterUniqueTogether(
            name="session",
            unique_together=set([("organization", "begin", "day")]),
        ),
        migrations.RemoveField(
            model_name="session",
            name="timeslot",
        ),
        migrations.DeleteModel(
            name="SessionTimeSlot",
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0015_userprofile_office_member"),
    ]

    operations = [
        migrations.AlterField(
            model_name="userprofile",
            name="status",
            field=models.PositiveSmallIntegerField(
                choices=[
                    (0, "---------"),
                    (10, "Actif"),
                    (20, "Réserve"),
                    (30, "Inactif"),
                    (40, "Archive"),
                ],
                verbose_name="Statut",
                default=0,
            ),
        ),
    ]



from django.db import migrations


def ws_to_vs(apps, schema_editor):
    Season = apps.get_model("challenge", "Season")
    # As of 2020-11-09, there are 3 Seasons with 'WS' (and they're alone as such)
    # As the Orga will move to "VS", these can safely be deleted.
    for season in Season.objects.all():
        if season.cantons == ["WS"]:
            season.delete()
            continue
        # Alter the cantons' array to set "VS" where it's "WS" (but this will affect no Season)
        cantons = ["VS" if c == "WS" else c for c in season.cantons]
        season.cantons = cantons
        season.save()


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0073_correct_season_state_labels"),
    ]

    operations = [
        migrations.RunPython(ws_to_vs, migrations.RunPython.noop),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib.auth import get_user_model
from django.test import Client
from django.utils.translation import activate

from allauth.account.models import EmailAddress
from faker import Faker
from rolepermissions.checkers import has_role
from rolepermissions.roles import assign_role

from apps.common import DV_STATES
from apps.user import FORMATION_M1, get_new_username
from apps.user.models import UserManagedState, UserProfile

fake = Faker()


class AuthClient(Client):
    role = None

    def __init__(self):
        super(AuthClient, self).__init__()
        self.USERNAME = get_new_username()
        self.PASSWORD = fake.password()
        self.EMAIL = fake.email()

        # Create that user
        self.user = get_user_model().objects.create_user(
            username=self.USERNAME,
            password=self.PASSWORD,
            email=self.EMAIL,
            first_name=fake.first_name(),
            last_name=fake.last_name(),
        )
        UserProfile.objects.get_or_create(user=self.user)
        # Create his trusted Email
        EmailAddress.objects.create(
            user=self.user, email=self.EMAIL, verified=True, primary=True
        )
        self.login(email=self.EMAIL, password=self.PASSWORD)

        self.language = "fr"
        activate(self.language)

        if self.role:
            assign_role(self.user, self.role)


class CollaboratorAuthClient(AuthClient):
    def __init__(self):
        super().__init__()
        self.user.profile.formation = FORMATION_M1
        self.user.profile.save()
        assert has_role(self.user, "collaborator")


class StateManagerAuthClient(AuthClient):
    role = "state_manager"

    def __init__(self):
        super(StateManagerAuthClient, self).__init__()
        # Make hir manager for one state
        UserManagedState.objects.get_or_create(user=self.user, canton=DV_STATES[0])


class PowerUserAuthClient(AuthClient):
    role = "power_user"


class CoordinatorAuthClient(AuthClient):
    role = "coordinator"


class SuperUserAuthClient(AuthClient):
    def __init__(self):
        super(SuperUserAuthClient, self).__init__()
        self.user.is_superuser = True
        self.user.save()



# Generated by Django 2.2.17 on 2021-01-27 14:52

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0067_role_reset_for_coordinator_access"),
    ]

    operations = [
        migrations.RenameField(
            model_name="userprofile",
            old_name="cresus_employee_number",
            new_name="employee_code",
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="employee_code",
            field=models.CharField(
                blank=True, max_length=63, verbose_name="Code salarié·e"
            ),
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2016 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.core.exceptions import PermissionDenied
from django.utils.functional import cached_property

from defivelo.roles import user_cantons

from ..models import Season


class CantonSeasonFormMixin(object):
    allow_season_fetch = False

    @cached_property
    def season_object(self):
        """
        Return the season object without any constraints
        """
        try:
            seasonpk = int(
                self.kwargs["seasonpk"]
                if "seasonpk" in self.kwargs
                else self.kwargs["pk"]
            )
            return Season.objects.prefetch_related("leader").get(pk=seasonpk)
        except (ValueError, KeyError):
            return None

    @cached_property
    def season(self):
        """
        DEPRECATED
        This is too complicated, unreliable, so shouldn't be used anymore.
        self.season a Season object of the currently running season, with restrictions
        """
        try:
            season = self.season_object

            if not season:
                return None

            # Check that the intersection isn't empty
            usercantons = user_cantons(self.request.user)
            if usercantons and not list(
                set(usercantons).intersection(set(season.cantons))
            ):
                # If the user is marked as state manager for that season
                if season.leader == self.request.user:
                    return season
                # Verify that this state manager can access that canton as mobile
                if (
                    list(
                        set(
                            [self.request.user.profile.affiliation_canton]
                            + self.request.user.profile.activity_cantons
                        ).intersection(set(season.cantons))
                    )
                    and not self.raise_without_cantons
                ):
                    raise LookupError
                raise PermissionDenied
        except LookupError:
            # That user doesn't manage cantons
            if not self.allow_season_fetch:
                season = None
        except KeyError:
            # We're looking at a list
            season = None
        except Season.DoesNotExist:
            # I can't see it, or it doesn't exist
            raise ValueError("Can't see season or it doesn't exist")
        return season

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs["season"] = self.season_object
        try:
            cantons = user_cantons(self.request.user)
        except LookupError:
            cantons = None
        if self.season and cantons:
            # Check that one canton is in the intersection
            cantons = list(set(cantons).intersection(set(self.season.cantons)))
        kwargs["cantons"] = cantons
        return kwargs



# Generated by Django 2.2.9 on 2020-03-04 11:46

from django.contrib.auth import get_user_model
from django.db import migrations
from rolepermissions.checkers import has_role
from rolepermissions.permissions import revoke_permission
from rolepermissions.roles import assign_role


def reset_power_roles(apps, schema_editor):
    User = get_user_model()
    for user in User.objects.all():
        if has_role(user, "power_user"):
            # Re-assigning the role will sync the default associated permissions
            assign_role(user, "power_user")


def revoke_settings_crud_from_power_users(apps, schema_editor):
    User = get_user_model()
    for user in User.objects.all():
        if has_role(user, "power_user"):
            revoke_permission(user, "settings_crud")
            user.save()


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0052_allow_power_user_to_set_cresus_number"),
    ]

    operations = [
        migrations.RunPython(reset_power_roles, revoke_settings_crud_from_power_users),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2020 Didier 'OdyX' Raboud <didier.raboud@liip.ch>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.core.exceptions import ValidationError
from django.db.utils import IntegrityError
from django.test import TestCase
from django.urls import reverse

from factory import fuzzy

from apps.challenge.models import AnnualStateSetting
from apps.common import DV_STATES
from defivelo.tests.utils import AuthClient, PowerUserAuthClient, StateManagerAuthClient

from .factories import AnnualStateSettingFactory
from .test_seasons import SeasonTestCaseMixin


class SettingTestCase(TestCase):
    def setUp(self):
        self.year = fuzzy.FuzzyInteger(1999, 2050).fuzz()
        self.canton = fuzzy.FuzzyChoice(DV_STATES).fuzz()

    def test_settings_no_negative_costs(self):
        s0 = AnnualStateSetting(
            year=self.year,
            canton=self.canton,
            cost_per_bike=fuzzy.FuzzyInteger(-100, -1).fuzz(),
        )
        with self.assertRaises(expected_exception=ValidationError):
            s0.full_clean()

        s1 = AnnualStateSetting(
            year=self.year,
            canton=self.canton,
            cost_per_participant=fuzzy.FuzzyInteger(-100, -1).fuzz(),
        )
        with self.assertRaises(expected_exception=ValidationError):
            s1.full_clean()

    def test_settings_unique_together(self):
        s0 = AnnualStateSetting(year=self.year, canton=self.canton)
        s0.save()
        s1 = AnnualStateSetting(year=self.year, canton=self.canton)
        with self.assertRaises(expected_exception=ValidationError):
            s1.full_clean()
        with self.assertRaisesMessage(
            expected_exception=IntegrityError,
            expected_message="duplicate key value violates unique constraint",
        ):
            s1.save()


class SettingsViewsTestCase(SeasonTestCaseMixin):
    def setUp(self):
        """
        Prepare the URLs
        """
        super().setUp()
        self.url_yearly = reverse(
            "annualstatesettings-list",
            kwargs={"year": fuzzy.FuzzyInteger(1999, 2050).fuzz()},
        )
        self.create_initial = {
            "year": fuzzy.FuzzyInteger(1999, 2050).fuzz(),
            "canton": fuzzy.FuzzyChoice(DV_STATES).fuzz(),
            "cost_per_bike": fuzzy.FuzzyInteger(0, 100).fuzz(),
            "cost_per_participant": fuzzy.FuzzyInteger(0, 100).fuzz(),
        }
        self.url_create = reverse(
            "annualstatesetting-create",
            kwargs={"year": self.create_initial["year"]},
        )

        self.setting = AnnualStateSettingFactory()
        self.setting.save()

        self.url_update = reverse(
            "annualstatesetting-update",
            kwargs={"year": self.setting.year, "pk": self.setting.pk},
        )
        self.restricted_urls = [self.url_yearly, self.url_create, self.url_update]


class AuthUserTest(SettingsViewsTestCase):
    def setUp(self):
        self.client = AuthClient()
        super().setUp()

    def test_settings_list_access(self):
        for url in self.restricted_urls:
            self.assertEqual(self.client.get(url).status_code, 403, url)


class StateManagerUserTest(SettingsViewsTestCase):
    def setUp(self):
        self.client = StateManagerAuthClient()
        super().setUp()

    def test_no_access_to_foreignsettings(self):
        # Make the setting _not_ an accessible setting
        foreignsetting = AnnualStateSettingFactory(
            canton=next(
                c
                for c in DV_STATES
                if c
                not in self.client.user.managedstates.all().values_list(
                    "canton", flat=True
                )
            )
        )
        foreignsetting.save()

        # The StateManager can't access other canton's settings
        url = reverse(
            "annualstatesetting-update",
            kwargs={"year": foreignsetting.year, "pk": foreignsetting.pk},
        )
        self.assertEqual(self.client.get(url).status_code, 404, url)

    def test_create_setting_in_our_cantons(self):
        # Now create one in our cantons, and ensure we return on the page that corresponds to our year
        initial = self.create_initial
        initial["year"] = 1968
        initial["canton"] = self.client.user.managedstates.first().canton
        self.assertRedirects(
            self.client.post(self.url_create, initial),
            reverse(
                "annualstatesettings-list",
                kwargs={"year": 1968},
            ),
            target_status_code=200,
        )

    def test_update_setting_in_our_cantons(self):
        setting = AnnualStateSettingFactory(
            canton=self.client.user.managedstates.first().canton
        )
        setting.save()

        update_url = reverse(
            "annualstatesetting-update",
            kwargs={"year": setting.year, "pk": setting.pk},
        )

        initial = setting.__dict__
        initial["year"] = 2021

        # On update, ensure we return on the page that corresponds to our new year
        self.assertRedirects(
            self.client.post(update_url, initial),
            reverse(
                "annualstatesettings-list",
                kwargs={"year": 2021},
            ),
            target_status_code=200,
        )


class PowerUserTest(SettingsViewsTestCase):
    def setUp(self):
        self.client = PowerUserAuthClient()
        super().setUp()

    def test_settings_list_access_power(self):
        # The restricted urls are allowed to power user
        for url in self.restricted_urls:
            self.assertEqual(self.client.get(url).status_code, 200, url)

        # On create, ensure we return on the page that corresponds to our year
        initial = self.create_initial
        initial["year"] = 1968
        self.assertRedirects(
            self.client.post(self.url_create, self.create_initial),
            reverse(
                "annualstatesettings-list",
                kwargs={"year": 1968},
            ),
            target_status_code=200,
        )
        initial = self.setting.__dict__
        initial["year"] = 1968

        # On update, ensure we return on the page that corresponds to our new year
        self.assertRedirects(
            self.client.post(self.url_update, initial),
            reverse(
                "annualstatesettings-list",
                kwargs={"year": 1968},
            ),
            target_status_code=200,
        )



# Generated by Django 2.2.9 on 2020-03-06 14:04

from django.contrib.auth import get_user_model
from django.db import migrations
from rolepermissions.checkers import has_role
from rolepermissions.permissions import revoke_permission
from rolepermissions.roles import assign_role


def reset_roles(apps, schema_editor):
    User = get_user_model()
    for user in User.objects.all():
        # Re-assigning roles will sync the default associated permissions
        if has_role(user, "power_user"):
            assign_role(user, "power_user")

        if has_role(user, "state_manager"):
            assign_role(user, "state_manager")


def revoke_perms(apps, schema_editor):
    User = get_user_model()
    for user in User.objects.all():
        if has_role(user, "power_user") or has_role(user, "state_manager"):
            revoke_permission(user, "timesheet_editor")


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0053_allow_power_user_to_crud_settings"),
    ]

    operations = [
        migrations.RunPython(reset_roles, revoke_perms),
    ]



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0010_auto_20151008_1613"),
    ]

    operations = [
        migrations.AlterField(
            model_name="userprofile",
            name="actor_for",
            field=models.ForeignKey(
                verbose_name="Intervenant",
                related_name="actors",
                null=True,
                blank=True,
                to="challenge.QualificationActivity",
                on_delete=models.CASCADE,
            ),
        ),
    ]



# Generated by Django 3.2.14 on 2023-09-05 08:02

from django.db import migrations, models

import apps.common.fields


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0072_alter_userprofile_languages_challenges"),
    ]

    operations = [
        migrations.AlterField(
            model_name="usermanagedstate",
            name="canton",
            field=models.CharField(
                choices=[
                    ("AG", "Aargau"),
                    ("AR", "Appenzell Ausserrhoden"),
                    ("BS", "Basel-Stadt"),
                    ("BL", "Basel-Land"),
                    ("BE", "Berne"),
                    ("FR", "Fribourg"),
                    ("GE", "Geneva"),
                    ("GR", "Graubuenden"),
                    ("JU", "Jura"),
                    ("LU", "Lucerne"),
                    ("NE", "Neuchatel"),
                    ("NW", "Nidwalden"),
                    ("OW", "Obwalden"),
                    ("SZ", "Schwyz"),
                    ("SO", "Solothurn"),
                    ("SG", "St. Gallen"),
                    ("VS", "Valais"),
                    ("VD", "Vaud"),
                    ("ZG", "Zug"),
                    ("ZH", "Zurich"),
                ],
                max_length=2,
                verbose_name="Canton",
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="activity_cantons",
            field=apps.common.fields.ChoiceArrayField(
                base_field=models.CharField(
                    choices=[
                        ("AG", "Aargau"),
                        ("AR", "Appenzell Ausserrhoden"),
                        ("BS", "Basel-Stadt"),
                        ("BL", "Basel-Land"),
                        ("BE", "Berne"),
                        ("FR", "Fribourg"),
                        ("GE", "Geneva"),
                        ("GR", "Graubuenden"),
                        ("JU", "Jura"),
                        ("LU", "Lucerne"),
                        ("NE", "Neuchatel"),
                        ("NW", "Nidwalden"),
                        ("OW", "Obwalden"),
                        ("SZ", "Schwyz"),
                        ("SO", "Solothurn"),
                        ("SG", "St. Gallen"),
                        ("VS", "Valais"),
                        ("VD", "Vaud"),
                        ("ZG", "Zug"),
                        ("ZH", "Zurich"),
                    ],
                    max_length=2,
                ),
                blank=True,
                default=list,
                size=None,
                verbose_name="Défi Vélo mobile",
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="affiliation_canton",
            field=models.CharField(
                blank=True,
                choices=[
                    ("", "---------"),
                    ("AG", "Aargau"),
                    ("AR", "Appenzell Ausserrhoden"),
                    ("BS", "Basel-Stadt"),
                    ("BL", "Basel-Land"),
                    ("BE", "Berne"),
                    ("FR", "Fribourg"),
                    ("GE", "Geneva"),
                    ("GR", "Graubuenden"),
                    ("JU", "Jura"),
                    ("LU", "Lucerne"),
                    ("NE", "Neuchatel"),
                    ("NW", "Nidwalden"),
                    ("OW", "Obwalden"),
                    ("SZ", "Schwyz"),
                    ("SO", "Solothurn"),
                    ("SG", "St. Gallen"),
                    ("VS", "Valais"),
                    ("VD", "Vaud"),
                    ("ZG", "Zug"),
                    ("ZH", "Zurich"),
                ],
                max_length=2,
                verbose_name="Canton d’affiliation",
            ),
        ),
    ]



# The MIT License (MIT)
#
# Copyright (c) 2015 Ha Pham
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
Models originally based on `simple-article==0.2.1`.
"""

from django.db import models
from django.utils import timezone
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _

from taggit.managers import TaggableManager
from tinymce.models import HTMLField


class Article(models.Model):
    """
    Simple article model with basic fields
    """

    title = models.CharField(_("Title"), max_length=255)
    slug = models.SlugField(_("Slug"), max_length=255, unique=True, blank=True)
    summary = models.TextField(_("Summary"), null=True, blank=True)
    image = models.ImageField(
        _("Image"), blank=True, null=True, upload_to="articles/%Y/%m/%d"
    )
    modified = models.DateTimeField(_("Modified"), default=timezone.now)
    published = models.BooleanField(default=False)
    body = HTMLField(_("Body"))
    tags = TaggableManager()

    class Meta:
        ordering = ["-modified"]
        verbose_name = "Article"

    def __str__(self):
        return "Article: %s" % self.title

    def save(self, *args, **kwargs):
        if not self.id:
            self.slug = slug = slugify(self.title)
            counter = 1
            while self.__class__.objects.filter(slug=self.slug).exists():
                self.slug = "{0}-{1}".format(slug, counter)
            counter += 1
        return super().save(*args, **kwargs)



from django.conf.urls import include
from django.urls import re_path, reverse_lazy
from django.utils import timezone
from django.views.decorators.cache import never_cache
from django.views.generic import RedirectView

from apps.salary.views import (
    CleanupOrphanedTimesheets,
    ExportMonthlyControl,
    ExportMonthlyTimesheets,
    RedirectUserMonthlyTimesheets,
    SendTimesheetsReminder,
    UserMonthlyTimesheets,
    ValidationsMonthView,
    ValidationsYearView,
    ValidationUpdate,
    YearlyTimesheets,
)

app_name = "salary"

urlpatterns = [
    re_path(
        r"^timesheets/",
        include(
            [
                re_path(
                    r"^$",
                    RedirectView.as_view(
                        url=reverse_lazy(
                            "salary:timesheets-overview",
                            kwargs={"year": timezone.now().year},
                        ),
                        permanent=False,
                    ),
                    name="timesheets",
                ),
                re_path(
                    r"^(?P<year>\d{4})/$",
                    YearlyTimesheets.as_view(),
                    name="timesheets-overview",
                ),
                re_path(
                    r"^(?P<year>[0-9]{4})-(?P<month>[0-9]+)/",
                    include(
                        [
                            re_path(
                                r"^$",
                                RedirectUserMonthlyTimesheets.as_view(),
                                name="my-timesheets",
                            ),
                            re_path(
                                r"^cleanup/$",
                                CleanupOrphanedTimesheets.as_view(),
                                name="cleanup-timesheets",
                            ),
                            re_path(
                                r"(?P<format>[a-z]+)-export$",
                                ExportMonthlyTimesheets.as_view(),
                                name="accounting-export",
                            ),
                            re_path(
                                r"(?P<format>[a-z]+)-control$",
                                ExportMonthlyControl.as_view(),
                                name="control-export",
                            ),
                            re_path(
                                r"^(?P<pk>[0-9]+)/$",
                                UserMonthlyTimesheets.as_view(),
                                name="user-timesheets",
                            ),
                            re_path(
                                r"^send_reminder/$",
                                SendTimesheetsReminder.as_view(),
                                name="send-timesheets-reminder",
                            ),
                        ]
                    ),
                ),
            ]
        ),
    ),
    # Validations
    re_path(
        r"^validations/(?P<year>[0-9]{4})/",
        include(
            [
                re_path(
                    r"^$",
                    never_cache(ValidationsYearView.as_view()),
                    name="validations-year",
                ),
                re_path(
                    r"^(?P<month>[0-9]{1,2})/",
                    include(
                        [
                            re_path(
                                r"^$",
                                never_cache(
                                    ValidationsMonthView.as_view(month_format="%m")
                                ),
                                name="validations-month",
                            ),
                            re_path(
                                r"^(?P<canton>[A-z]{2})/$",
                                never_cache(ValidationUpdate.as_view()),
                                name="validation-update",
                            ),
                        ]
                    ),
                ),
            ]
        ),
    ),
]



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0024_auto_20151015_0958"),
    ]

    operations = [
        migrations.AddField(
            model_name="session",
            name="place",
            field=models.CharField(
                max_length=512, verbose_name="Lieu de la qualification", blank=True
            ),
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015, 2016 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib.sites.models import Site
from django.forms import Form as DjangoEmptyForm
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views.generic.edit import FormView

from rolepermissions.mixins import HasPermissionsMixin

from .mixins import ProfileMixin


class UserDelete(ProfileMixin, HasPermissionsMixin, FormView):
    template_name = "delete/user_delete.html"
    success_message = _("Utilisateur·rice supprimé·e")
    form_class = DjangoEmptyForm
    required_permission = "user_deletions"
    cantons = False

    def get_context_data(self, **kwargs):
        context = super(UserDelete, self).get_context_data(**kwargs)
        # Add our menu_category context
        context["userprofile"] = self.get_object()
        context["current_site"] = Site.objects.get_current()
        context["login_uri"] = self.request.build_absolute_uri(reverse("account_login"))
        return context

    def form_valid(self, form):
        user = self.get_object()
        user.profile.delete()
        return super(UserDelete, self).form_valid(form)



# Generated by Django 1.11.11 on 2018-03-16 16:05
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0046_auto_20180316_1657"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="marital_status",
            field=models.PositiveSmallIntegerField(
                choices=[
                    (0, "---------"),
                    (10, "C\xe9libataire"),
                    (20, "Mari\xe9\xb7e"),
                    (30, "Divorc\xe9\xb7e"),
                    (40, "Veu\xb7f\xb7ve"),
                    (50, "En partenariat enregistr\xe9"),
                ],
                default=0,
                verbose_name="\xc9tat civil",
            ),
        ),
    ]



# Generated by Django 1.11.11 on 2018-03-16 15:57
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0045_allow_power_user_to_mark_inactive"),
    ]

    operations = [
        migrations.AlterField(
            model_name="userprofile",
            name="bagstatus",
            field=models.PositiveSmallIntegerField(
                choices=[
                    (0, "---"),
                    (10, "En pr\xeat"),
                    (20, "Pay\xe9"),
                    (30, "Offert"),
                ],
                default=0,
                verbose_name="Sac D\xe9fi V\xe9lo",
            ),
        ),
    ]



from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("challenge", "0018_season"),
    ]

    operations = [
        migrations.CreateModel(
            name="HelperSessionAvailability",
            fields=[
                (
                    "id",
                    models.AutoField(
                        serialize=False,
                        verbose_name="ID",
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                (
                    "availability",
                    models.CharField(
                        max_length=1,
                        choices=[("y", "Oui"), ("i", "Si nécessaire"), ("n", "Non")],
                        verbose_name="Disponible",
                    ),
                ),
                (
                    "helper",
                    models.ForeignKey(
                        verbose_name="Moniteur",
                        to=settings.AUTH_USER_MODEL,
                        related_name="availabilities",
                        on_delete=models.CASCADE,
                    ),
                ),
                (
                    "session",
                    models.ForeignKey(
                        verbose_name="Session",
                        to="challenge.Session",
                        related_name="availabilities",
                        on_delete=models.CASCADE,
                    ),
                ),
            ],
            options={
                "verbose_name": "Disponibilité par session",
                "verbose_name_plural": "Disponibilités par session",
                "ordering": ["session", "helper", "availability"],
            },
        ),
        migrations.AlterModelOptions(
            name="season",
            options={
                "verbose_name": "Saison",
                "verbose_name_plural": "Saisons",
                "ordering": ["begin", "end"],
            },
        ),
    ]



# Generated by Django 1.11.4 on 2017-08-30 13:17
from __future__ import unicode_literals

from django.db import migrations, models


def vsow_to_hw(apps, schema_editor):
    # We can't import the Season model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    Season = apps.get_model("challenge", "Season")
    for season in Season.objects.all():
        cantons = ["WS" if c == "VS-OW" else c for c in season.cantons]
        season.cantons = cantons
        season.save()


def hw_to_vsow(apps, schema_editor):
    # We can't import the Season model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    Season = apps.get_model("challenge", "Season")
    for season in Season.objects.all():
        cantons = ["VS-OW" if c == "WS" else c for c in season.cantons]
        season.cantons = cantons
        season.save()


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0042_auto_20170829_1438"),
    ]

    operations = [
        migrations.RunPython(vsow_to_hw, hw_to_vsow),
        migrations.AlterField(
            model_name="season",
            name="cantons",
            field=models.CharField(
                choices=[
                    ("AR", "Appenzell Ausserrhoden"),
                    ("BE", "Berne"),
                    ("BS", "Basel-Stadt"),
                    ("FR", "Fribourg"),
                    ("GE", "Geneva"),
                    ("JU", "Jura"),
                    ("LU", "Lucerne"),
                    ("NE", "Neuchatel"),
                    ("SG", "St. Gallen"),
                    ("VD", "Vaud"),
                    ("VS", "Valais"),
                    ("WS", "Haut-Valais"),
                    ("ZH", "Zurich"),
                ],
                max_length=38,
                verbose_name="Cantons",
            ),
        ),
    ]



# Generated by Django 1.11.4 on 2017-08-30 14:00
from __future__ import unicode_literals

from django.db import migrations, models


def vsow_to_hw(apps, schema_editor):
    Address = apps.get_model("common", "Address")
    for address in Address.objects.all():
        if address.address_canton == "VS-OW":
            address.address_canton = "WS"
            address.save()


def hw_to_vsow(apps, schema_editor):
    Address = apps.get_model("common", "Address")
    for address in Address.objects.all():
        if address.address_canton == "WS":
            address.address_canton = "VS-OW"
            address.save()


class Migration(migrations.Migration):
    dependencies = [
        ("common", "0004_auto_20170201_0823"),
    ]

    operations = [
        migrations.RunPython(vsow_to_hw, hw_to_vsow),
        migrations.AlterField(
            model_name="address",
            name="address_canton",
            field=models.CharField(blank=True, max_length=2, verbose_name="Canton"),
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015,2020 Didier Raboud <didier.raboud@liip.ch>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib.messages.views import SuccessMessageMixin
from django.core.exceptions import PermissionDenied
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from django.views.generic.edit import CreateView, DeleteView, UpdateView

from defivelo.roles import has_permission

from ..forms import QualificationDeleteForm, QualificationForm
from ..models import Qualification, Session
from .session import SessionMixin


class QualiMixin(SessionMixin):
    model = Qualification
    context_object_name = "qualification"
    form_class = QualificationForm
    view_does_crud = True

    def get_session_pk(self):
        resolvermatch = self.request.resolver_match
        if "sessionpk" in resolvermatch.kwargs:
            return int(resolvermatch.kwargs["sessionpk"])

    def get_season_pk(self):
        resolvermatch = self.request.resolver_match
        if "seasonpk" in resolvermatch.kwargs:
            return int(resolvermatch.kwargs["seasonpk"])

    def get_success_url(self):
        return reverse_lazy(
            "session-detail",
            kwargs={"seasonpk": self.get_season_pk(), "pk": self.get_session_pk()},
        )

    def get_form_kwargs(self):
        form_kwargs = super().get_form_kwargs()
        try:
            session = Session.objects.get(pk=self.get_session_pk())
            form_kwargs["session"] = session
            # Coordinator without StateManager rights
            form_kwargs["is_for_coordinator"] = (
                not has_permission(self.request.user, "challenge_session_crud")
                and self.request.user == session.orga.coordinator
            )
        except Exception:
            pass
        return form_kwargs

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        # Add our menu_category context
        context["menu_category"] += " qualification"
        try:
            context["session"] = Session.objects.get(pk=self.get_session_pk())
        except Exception:
            pass
        context["season"] = self.season_object
        context["qualification_user_errors"] = (
            self.object.user_errors(self.request.user) if self.object else []
        )
        return context


class QualiCreateView(QualiMixin, SuccessMessageMixin, CreateView):
    success_message = _("Qualif’ créée")

    def get_initial(self):
        return {"session": self.get_session_pk()}

    def dispatch(self, *args, **kwargs):
        """
        Quali Creation is only for StateManagers, not coordinators
        """
        if not has_permission(self.request.user, "challenge_session_crud"):
            raise PermissionDenied
        return super().dispatch(*args, **kwargs)


class QualiUpdateView(QualiMixin, SuccessMessageMixin, UpdateView):
    success_message = _("Qualif’ mise à jour")

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["timesheets"] = self.object.get_related_timesheets().distinct()
        return context


class QualiDeleteView(QualiMixin, DeleteView):
    success_message = _("Qualif’ supprimée")
    form_class = QualificationDeleteForm

    def dispatch(self, *args, **kwargs):
        """
        Quali Deletion is only for StateManagers, not coordinators
        """
        if not has_permission(self.request.user, "challenge_session_crud"):
            raise PermissionDenied
        return super().dispatch(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["timesheets"] = self.object.get_related_timesheets().distinct()
        return context

    def get_form_kwargs(self):
        # Cleanup unused kwargs genereated by QualiMixin
        return {
            k: v
            for (k, v) in super().get_form_kwargs().items()
            if k not in ["season", "cantons", "session", "is_for_coordinator"]
        }

    def form_valid(self, *args, **kwargs):
        self.get_object().get_related_timesheets().delete()
        return super().form_valid(*args, **kwargs)



# Generated by Django 1.11.4 on 2017-08-29 12:38
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0041_seasons_are_seasons"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="season",
            options={
                "ordering": ["year", "season"],
                "verbose_name": "Saison",
                "verbose_name_plural": "Saisons",
            },
        ),
        migrations.RemoveField(
            model_name="season",
            name="begin",
        ),
        migrations.RemoveField(
            model_name="season",
            name="end",
        ),
    ]



# Generated by Django 2.2.16 on 2020-11-02 17:39

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("common", "0006_unaccented_psql"),
    ]

    operations = [
        migrations.AlterField(
            model_name="address",
            name="address_additional",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="Complément d’adresse"
            ),
        ),
    ]



from django.test import TestCase
from django.urls import reverse
from django.utils import timezone

from apps.challenge.tests.factories import QualificationFactory, SessionFactory

freeforallurls = ["public-nextqualifs", "public-json-nextqualifs"]


class AnonymousUserTest(TestCase):
    def setUp(self):
        self.session = SessionFactory(day=timezone.localdate())
        self.qualifs = QualificationFactory.create_batch(size=3, session=self.session)

    def test_access_to_qualifs_list(self):
        for symbolicurl in freeforallurls:
            url = reverse(symbolicurl)
            # Final URL is OK
            response = self.client.get(url, follow=True)
            self.assertEqual(response.status_code, 200, url)
            self.assertIn(self.session.city.encode(), response.content)



from django.db import migrations
from django.db.models import Count, Q
from django.utils import timezone


def update_timesheet_leader_count(session_model, timesheet):
    session_qs = (
        # Code duplicated from `apps.salary.views.UserMonthlyTimesheets.get_queryset`
        (
            session_model.objects.values("day")
            .filter(
                Q(qualifications__actor=timesheet.user)
                | Q(qualifications__helpers=timesheet.user)
                | Q(qualifications__leader=timesheet.user)
            )
            .filter(day__lte=timezone.now())
            .annotate(
                orga_count=Count("orga_id", distinct=True),
                helper_count=Count(
                    "qualifications__pk",
                    filter=Q(qualifications__helpers=timesheet.user)
                    | Q(qualifications__leader=timesheet.user),
                    distinct=True,
                ),
                leader_count=Count(
                    "qualifications__pk",
                    filter=Q(qualifications__leader=timesheet.user),
                    distinct=True,
                ),
                actor_count=Count(
                    "qualifications__pk",
                    filter=Q(qualifications__actor=timesheet.user),
                    distinct=True,
                ),
            )
        )
        .exclude(actor_count=0, helper_count=0)
        .order_by("day")
    )
    session = session_qs.get(day=timesheet.date)
    timesheet.leader_count = session["leader_count"]
    timesheet.save()


def update_timesheets(apps, schema_editor):
    Session = apps.get_model("challenge", "Session")
    Timesheet = apps.get_model("salary", "Timesheet")
    for timesheet in Timesheet.objects.all():
        update_timesheet_leader_count(Session, timesheet)


class Migration(migrations.Migration):
    dependencies = [
        ("salary", "0005_timesheet_leader_count"),
    ]

    operations = [
        migrations.RunPython(update_timesheets),
    ]



# Generated by Django 2.2.8 on 2019-12-12 13:54

import django.db.models.deletion
import parler.fields
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0052_auto_20190225_1631"),
    ]

    operations = [
        migrations.AlterField(
            model_name="helperseasonworkwish",
            name="helper",
            field=models.ForeignKey(
                limit_choices_to={"profile__isnull": False},
                on_delete=django.db.models.deletion.CASCADE,
                related_name="work_wishes",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Moniteur",
            ),
        ),
        migrations.AlterField(
            model_name="helpersessionavailability",
            name="helper",
            field=models.ForeignKey(
                limit_choices_to={"profile__isnull": False},
                on_delete=django.db.models.deletion.CASCADE,
                related_name="availabilities",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Moniteur",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhelperseasonworkwish",
            name="helper",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"profile__isnull": False},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Moniteur",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhelpersessionavailability",
            name="helper",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"profile__isnull": False},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Moniteur",
            ),
        ),
        migrations.AlterField(
            model_name="historicalqualification",
            name="activity_A",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"category": "A"},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="challenge.QualificationActivity",
                verbose_name="Agilité",
            ),
        ),
        migrations.AlterField(
            model_name="historicalqualification",
            name="activity_B",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"category": "B"},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="challenge.QualificationActivity",
                verbose_name="Mécanique",
            ),
        ),
        migrations.AlterField(
            model_name="historicalqualification",
            name="activity_C",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"category": "C"},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="challenge.QualificationActivity",
                verbose_name="Rencontre",
            ),
        ),
        migrations.AlterField(
            model_name="historicalqualification",
            name="actor",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"profile__actor_for__isnull": False},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Intervenant",
            ),
        ),
        migrations.AlterField(
            model_name="historicalqualification",
            name="leader",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to=models.Q(profile__formation="M2"),
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Moniteur 2",
            ),
        ),
        migrations.AlterField(
            model_name="historicalseason",
            name="leader",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"managedstates__isnull": False},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Chargé de projet",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsession",
            name="orga",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"address_canton__isnull": False, "status__in": [10]},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="orga.Organization",
                verbose_name="Établissement",
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="activity_A",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"category": "A"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="qualifs_A",
                to="challenge.QualificationActivity",
                verbose_name="Agilité",
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="activity_B",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"category": "B"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="qualifs_B",
                to="challenge.QualificationActivity",
                verbose_name="Mécanique",
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="activity_C",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"category": "C"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="qualifs_C",
                to="challenge.QualificationActivity",
                verbose_name="Rencontre",
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="actor",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"profile__actor_for__isnull": False},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="qualifs_actor",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Intervenant",
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="helpers",
            field=models.ManyToManyField(
                blank=True,
                limit_choices_to=models.Q(profile__formation__in=["M1", "M2"]),
                related_name="qualifs_mon1",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Moniteurs 1",
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="leader",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to=models.Q(profile__formation="M2"),
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="qualifs_mon2",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Moniteur 2",
            ),
        ),
        migrations.AlterField(
            model_name="qualificationactivitytranslation",
            name="master",
            field=parler.fields.TranslationsForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="translations",
                to="challenge.QualificationActivity",
            ),
        ),
        migrations.AlterField(
            model_name="season",
            name="leader",
            field=models.ForeignKey(
                limit_choices_to={"managedstates__isnull": False},
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
                verbose_name="Chargé de projet",
            ),
        ),
        migrations.AlterField(
            model_name="session",
            name="orga",
            field=models.ForeignKey(
                limit_choices_to={"address_canton__isnull": False, "status__in": [10]},
                on_delete=django.db.models.deletion.CASCADE,
                related_name="sessions",
                to="orga.Organization",
                verbose_name="Établissement",
            ),
        ),
    ]



from django.conf import settings
from django.contrib import admin
from django.templatetags.static import static
from django.utils.safestring import mark_safe

from parler.admin import TranslatableAdmin

from .models import MonthlyCantonalValidationUrl


class MonthlyCantonalValidationUrlAdmin(TranslatableAdmin):
    list_display = ["url", "name", "missing_languages"]
    list_display_links = (
        "url",
        "name",
    )
    search_fields = ("translations__name", "pk")

    def missing_languages(self, obj):
        missing_languages = list(
            set([lang[0] for lang in settings.LANGUAGES])
            - set(obj.get_available_languages())
        )
        if missing_languages:
            return " | ".join(missing_languages)
        else:
            return mark_safe('<img src="%s" />' % static("admin/img/icon-yes.svg"))


admin.site.register(MonthlyCantonalValidationUrl, MonthlyCantonalValidationUrlAdmin)



# Generated by Django 1.11.23 on 2019-12-11 14:11

"""
Replaces migrations from `simple-article==0.2.1`
"""

from __future__ import unicode_literals

import taggit.managers
import tinymce.models
from django.db import migrations, models
from django.utils import timezone

# fmt: off


class Migration(migrations.Migration):

    replaces = [('article', '0001_initial'), ('article', '0002_article_tags')]

    initial = True

    dependencies = [
        ('taggit', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Article',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('slug', models.SlugField(unique=True, max_length=255, verbose_name='Slug', blank=True)),
                ('summary', models.TextField(null=True, verbose_name='Summary', blank=True)),
                ('image', models.ImageField(upload_to='articles/%Y/%m/%d', null=True, verbose_name='Image', blank=True)),
                ('modified', models.DateTimeField(default=timezone.now, verbose_name='Modified')),
                ('published', models.BooleanField(default=False)),
                ('body', tinymce.models.HTMLField(verbose_name='Body')),
                ('tags', taggit.managers.TaggableManager(to='taggit.Tag', through='taggit.TaggedItem', help_text='A comma-separated list of tags.', verbose_name='Tags'))
            ],
            options={
                'ordering': ['-modified'],
                'verbose_name': 'Article',
            },
        ),
    ]

# fmt: on



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import glob
import os


def import_env_vars(directory):
    """
    List the files present in the given directory and for each of them create
    an environment variable named after the file, and which value is the
    contents of the file.
    """
    env_vars = glob.glob(os.path.join(directory, "*"))

    for env_var in env_vars:
        if os.path.isfile(env_var):
            with open(env_var, "r") as env_var_file:
                os.environ.setdefault(
                    env_var.split(os.sep)[-1], env_var_file.read().strip()
                )


def get_project_root_path():
    """
    Return the absolute path to the root of the project.
    """
    return os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir))



# Generated by Django 4.2.6 on 2025-09-09 14:01

from django.db import migrations, models


class Migration(migrations.Migration):
    """
    The E.164 standard itself caps the national significant number at 15 digits, but libphonenumber supports up to 17 digits in practice because some numbering plans exceed the formal limit
    """
    dependencies = [
        ("user", "0074_alter_userprofile_iban"),
    ]

    operations = [
        migrations.AlterField(
            model_name="userprofile",
            name="natel",
            field=models.CharField(blank=True, max_length=17),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="phone",
            field=models.CharField(blank=True, max_length=17, verbose_name="Téléphone"),
        ),
    ]



# Generated by Django 2.2.9 on 2020-03-02 11:01

import django.core.validators
import django.db.models.deletion
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("challenge", "0054_auto_20200114_1819"),
    ]

    operations = [
        migrations.CreateModel(
            name="HistoricalAnnualStateSetting",
            fields=[
                (
                    "id",
                    models.IntegerField(
                        auto_created=True, blank=True, db_index=True, verbose_name="ID"
                    ),
                ),
                ("canton", models.CharField(max_length=2, verbose_name="Canton")),
                ("year", models.PositiveSmallIntegerField(verbose_name="Année")),
                (
                    "cost_per_bike",
                    models.DecimalField(
                        decimal_places=2,
                        default=0,
                        help_text="CHF",
                        max_digits=8,
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Coût par vélo",
                    ),
                ),
                (
                    "cost_per_participant",
                    models.DecimalField(
                        decimal_places=2,
                        default=0,
                        help_text="CHF",
                        max_digits=8,
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Coût par participant",
                    ),
                ),
                ("history_id", models.AutoField(primary_key=True, serialize=False)),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Configuration cantonale par année",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="AnnualStateSetting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("canton", models.CharField(max_length=2, verbose_name="Canton")),
                ("year", models.PositiveSmallIntegerField(verbose_name="Année")),
                (
                    "cost_per_bike",
                    models.DecimalField(
                        decimal_places=2,
                        default=0,
                        help_text="CHF",
                        max_digits=8,
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Coût par vélo",
                    ),
                ),
                (
                    "cost_per_participant",
                    models.DecimalField(
                        decimal_places=2,
                        default=0,
                        help_text="CHF",
                        max_digits=8,
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Coût par participant",
                    ),
                ),
            ],
            options={
                "verbose_name": "Configuration cantonale par année",
                "verbose_name_plural": "Configurations cantonales par année",
                "unique_together": {("canton", "year")},
            },
        ),
    ]



# Generated by Django 2.2.9 on 2020-03-05 12:56

import django.core.validators
import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("orga", "0006_auto_20170831_1201"),
        ("challenge", "0056_invoice_permissions"),
    ]

    operations = [
        migrations.CreateModel(
            name="Invoice",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "generated_at",
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
                (
                    "status",
                    models.IntegerField(
                        choices=[(0, "Brouillon"), (1, "Validée")], default=0
                    ),
                ),
                (
                    "ref",
                    models.CharField(
                        max_length=20, unique=True, verbose_name="Référence"
                    ),
                ),
                (
                    "title",
                    models.CharField(blank=True, max_length=255, verbose_name="Titre"),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="invoices",
                        to="orga.Organization",
                        verbose_name="Établissement",
                    ),
                ),
                (
                    "season",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="challenge.Season",
                        verbose_name="Saison",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="InvoiceLine",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("nb_bikes", models.PositiveSmallIntegerField()),
                ("nb_participants", models.PositiveSmallIntegerField()),
                (
                    "cost_bikes",
                    models.DecimalField(
                        decimal_places=2,
                        max_digits=8,
                        validators=[django.core.validators.MinValueValidator(0)],
                    ),
                ),
                (
                    "cost_participants",
                    models.DecimalField(
                        decimal_places=2,
                        max_digits=8,
                        validators=[django.core.validators.MinValueValidator(0)],
                    ),
                ),
                (
                    "invoice",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="lines",
                        to="challenge.Invoice",
                    ),
                ),
                (
                    "session",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="challenge.Session",
                    ),
                ),
            ],
            options={
                "unique_together": {("session", "invoice")},
            },
        ),
    ]



# Generated by Django 2.2.9 on 2020-03-04 08:39

from django.contrib.auth import get_user_model
from django.db import migrations
from rolepermissions.checkers import has_role
from rolepermissions.permissions import revoke_permission
from rolepermissions.roles import assign_role


def reset_power_roles(apps, schema_editor):
    User = get_user_model()
    for user in User.objects.all():
        if has_role(user, "power_user"):
            # Re-assigning the role will sync the default associated permissions
            assign_role(user, "power_user")


def revoke_user_edit_cresus_employee_number_from_power_users(apps, schema_editor):
    User = get_user_model()
    for user in User.objects.all():
        if has_role(user, "power_user"):
            revoke_permission(user, "user_edit_cresus_employee_number")
            user.save()


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0051_userprofile_cresus_employee_number"),
    ]

    operations = [
        migrations.RunPython(
            reset_power_roles, revoke_user_edit_cresus_employee_number_from_power_users
        ),
    ]



# Generated by Django 2.2.16 on 2020-10-27 16:19

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0066_invoicelines_with_nullable_sessions"),
        ("challenge", "0066_add_schwyz"),
    ]

    operations = []



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("common", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="address",
            name="address_canton",
            field=models.CharField(max_length=2, blank=True, verbose_name="Canton"),
        ),
        migrations.AlterField(
            model_name="address",
            name="address_city",
            field=models.CharField(max_length=64, blank=True, verbose_name="Ville"),
        ),
        migrations.AlterField(
            model_name="address",
            name="address_no",
            field=models.CharField(max_length=8, blank=True, verbose_name="N°"),
        ),
        migrations.AlterField(
            model_name="address",
            name="address_street",
            field=models.CharField(max_length=255, blank=True, verbose_name="Rue"),
        ),
        migrations.AlterField(
            model_name="address",
            name="address_zip",
            field=models.CharField(max_length=4, blank=True, verbose_name="NPA"),
        ),
    ]



from django.forms.widgets import ChoiceWidget

from bootstrap3.renderers import FieldRenderer, InlineFieldRenderer
from bootstrap3.utils import add_css_class


class DvFieldRendererMixin:
    def get_form_group_class(self):
        """
        Override to avoid applying `required_css_class` on Selects with no empty choice
        """
        form_group_class = self.form_group_class
        if self.field.errors:
            if self.error_css_class:
                form_group_class = add_css_class(form_group_class, self.error_css_class)
        else:
            if self.field.form.is_bound:
                form_group_class = add_css_class(
                    form_group_class, self.success_css_class
                )
        widget = self.field.field.widget
        if (
            self.field.field.required
            and self.required_css_class
            and not (
                # `widget` is a non-multiple select with no empty value
                widget
                and isinstance(widget, ChoiceWidget)
                and not widget.allow_multiple_selected
                and all(
                    val not in self.field.field.empty_values
                    for val, _ in self.field.field.choices
                )
            )
        ):
            form_group_class = add_css_class(form_group_class, self.required_css_class)
        if self.layout == "horizontal":
            form_group_class = add_css_class(
                form_group_class, self.get_size_class(prefix="form-group")
            )
        return form_group_class


class DvFieldRenderer(DvFieldRendererMixin, FieldRenderer):
    pass


class DvInlineFieldRenderer(DvFieldRendererMixin, InlineFieldRenderer):
    pass



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Session",
            fields=[
                (
                    "id",
                    models.AutoField(
                        serialize=False,
                        verbose_name="ID",
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("day", models.DateField(blank=True, verbose_name="Date")),
            ],
            options={
                "verbose_name_plural": "Sessions",
                "verbose_name": "Session",
            },
        ),
        migrations.CreateModel(
            name="SessionTimeSlot",
            fields=[
                (
                    "id",
                    models.AutoField(
                        serialize=False,
                        verbose_name="ID",
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                ("begin", models.TimeField(verbose_name="Début")),
                ("end", models.TimeField(verbose_name="Fin")),
            ],
            options={
                "verbose_name_plural": "Horaires pour sessions",
                "ordering": ["begin", "end"],
                "verbose_name": "Horaire pour sessions",
            },
        ),
        migrations.AddField(
            model_name="session",
            name="timeslot",
            field=models.ForeignKey(
                blank=True,
                related_name="sessions",
                to="challenge.SessionTimeSlot",
                on_delete=models.CASCADE,
            ),
        ),
    ]



# Generated by Django 2.2.9 on 2020-03-04 13:20

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("salary", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="timesheet",
            name="validated_by",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0015_auto_20150907_1155"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="session",
            options={
                "ordering": ["day", "begin", "organization__name"],
                "verbose_name": "Session",
                "verbose_name_plural": "Sessions",
            },
        ),
    ]
